@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -212,6 +212,7 @@ usage(int exitStatus = EXIT_FAILURE)
 "                      (default: " << opts.up << ")\n" <<
 "\n" <<
 "    -v                verbose (print timing and diagnostics)\n" <<
+"    -version          print version information and exit\n" <<
 "    -h, -help         print this usage message and exit\n" <<
 "\n" <<
 "Level set options:\n" <<
@@ -591,6 +592,12 @@ main(int argc, char *argv[])
                 opts.up = strToVec3d(argv[i]);
             } else if (arg == "-v") {
                 opts.verbose = true;
+            } else if (arg == "-version" || arg == "--version") {
+                std::cout << "OpenVDB library version: "
+                    << openvdb::getLibraryVersionString() << "\n";
+                std::cout << "OpenVDB file format version: "
+                    << openvdb::OPENVDB_FILE_VERSION << std::endl;
+                return EXIT_SUCCESS;
             } else if (arg == "-h" || arg == "-help" || arg == "--help") {
                 usage(EXIT_SUCCESS);
             } else {
@@ -714,6 +721,6 @@ main(int argc, char *argv[])
     return retcode;
 }
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
