@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -44,6 +44,7 @@
 #include <string>
 #include <boost/numeric/conversion/conversion_traits.hpp>
 #include <boost/math/special_functions/cbrt.hpp>
+#include <boost/math/special_functions/fpclassify.hpp> // boost::math::isfinite
 #include <boost/random/mersenne_twister.hpp> // for boost::random::mt19937
 #include <boost/random/uniform_01.hpp>
 #include <boost/random/uniform_int.hpp>
@@ -54,6 +55,8 @@
 
 // Compile pragmas
 
+#define PRAGMA(x) _Pragma(#x)
+
 // Intel(r) compiler fires remark #1572: floating-point equality and inequality
 // comparisons are unrealiable when == or != is used with floating point operands.
 #if defined(__INTEL_COMPILER)
@@ -62,6 +65,12 @@
         _Pragma("warning (disable:1572)")
     #define OPENVDB_NO_FP_EQUALITY_WARNING_END \
         _Pragma("warning (pop)")
+#elif defined(__clang__)
+    #define OPENVDB_NO_FP_EQUALITY_WARNING_BEGIN \
+        PRAGMA(clang diagnostic push) \
+        PRAGMA(clang diagnostic ignored "-Wfloat-equal")
+    #define OPENVDB_NO_FP_EQUALITY_WARNING_END \
+        PRAGMA(clang diagnostic pop)
 #else
     // For GCC, #pragma GCC diagnostic ignored "-Wfloat-equal"
     // isn't working until gcc 4.2+,
@@ -236,7 +245,7 @@ template<typename Type>
 inline Type
 Clamp(Type x, Type min, Type max)
 {
-    assert(min<max);
+    assert( !(min>max) );
     return x > min ? x < max ? x : max : min;
 }
 
@@ -295,6 +304,7 @@ inline double Abs(double x) { return fabs(x); }
 inline long double Abs(long double x) { return fabsl(x); }
 inline uint32_t Abs(uint32_t i) { return i; }
 inline uint64_t Abs(uint64_t i) { return i; }
+inline bool Abs(bool b) { return b; }
 // On OSX size_t and uint64_t are different types
 #if defined(__APPLE__) || defined(MACOSX)
 inline size_t Abs(size_t i) { return i; }
@@ -347,6 +357,12 @@ isNegative(const Type& x) { return x < zeroVal<Type>(); }
 template<> inline bool isNegative<bool>(const bool&) { return false; }
 
 
+/// Return @c true if @a x is finite.
+template<typename Type>
+inline bool
+isFinite(const Type& x) { return boost::math::isfinite(x); }
+
+
 /// @brief Return @c true if @a a is equal to @a b to within
 /// the default floating-point comparison tolerance.
 template<typename Type>
@@ -447,8 +463,8 @@ doubleToInt64(const double aDoubleValue)
 
 // aUnitsInLastPlace is the allowed difference between the least significant digits
 // of the numbers' floating point representation
-// Please read refernce paper before trying to use isUlpsEqual
-// http://www.cygnus-software.com/papers/comparingFloats/comparingFloats.htm
+// Please read the reference paper before trying to use isUlpsEqual
+// http://www.cygnus-software.com/papers/comparingfloats/comparingfloats.htm
 inline bool
 isUlpsEqual(const double aLeft, const double aRight, const int64_t aUnitsInLastPlace)
 {
@@ -846,7 +862,7 @@ struct promote {
 
 /// @brief Return the index [0,1,2] of the smallest value in a 3D vector.
 /// @note This methods assumes operator[] exists and avoids branching.
-/// @details If two components of the input vector are equal and smaller then the
+/// @details If two components of the input vector are equal and smaller than the
 /// third component, the largest index of the two is always returned.
 /// If all three vector components are equal the largest index, i.e. 2, is
 /// returned. In other words the return value corresponds to the largest index
@@ -867,11 +883,11 @@ MinIndex(const Vec3T& v)
 
 /// @brief Return the index [0,1,2] of the largest value in a 3D vector.
 /// @note This methods assumes operator[] exists and avoids branching.
-/// @details If two components of the input vector are equal and larger then the
+/// @details If two components of the input vector are equal and larger than the
 /// third component, the largest index of the two is always returned.
 /// If all three vector components are equal the largest index, i.e. 2, is
 /// returned. In other words the return value corresponds to the largest index
-/// of the of the largest vector components.
+/// of the largest vector components.
 template<typename Vec3T>
 size_t
 MaxIndex(const Vec3T& v)
@@ -891,6 +907,6 @@ MaxIndex(const Vec3T& v)
 
 #endif // OPENVDB_MATH_MATH_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
