@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -29,7 +29,13 @@
 ///////////////////////////////////////////////////////////////////////////
 //
 /// @author Ken Museth
+///
 /// @file Stencils.h
+///
+/// @brief Defines various finite difference stencils by means of the
+///        "curiously recurring template pattern" on a BaseStencil
+///        that caches stencil values and stores a ValueAccessor for
+///        fast lookup.
 
 #ifndef OPENVDB_MATH_STENCILS_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_STENCILS_HAS_BEEN_INCLUDED
@@ -40,6 +46,7 @@
 #include <openvdb/Types.h>                 // for Real
 #include <openvdb/math/Coord.h>            // for Coord
 #include <openvdb/math/FiniteDifference.h> // for WENO5 and GudonovsNormSqrd
+#include <openvdb/tree/ValueAccessor.h>
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -48,18 +55,17 @@ namespace math {
 
 
 ////////////////////////////////////////
-
-
-template<typename _GridType, typename StencilType>
+    
+template<typename DerivedType, typename GridT, bool IsSafe>
 class BaseStencil
 {
 public:
-    typedef _GridType                        GridType;
-    typedef typename GridType::TreeType      TreeType;
-    typedef typename GridType::ValueType     ValueType;
-    typedef std::vector<ValueType>           BufferType;
-    typedef typename BufferType::iterator    IterType;
-    typedef typename GridType::ConstAccessor AccessorType;
+    typedef GridT                                       GridType;
+    typedef typename GridT::TreeType                    TreeType;
+    typedef typename GridT::ValueType                   ValueType;
+    typedef tree::ValueAccessor<const TreeType, IsSafe> AccessorType;
+    typedef std::vector<ValueType>                      BufferType;
+    typedef typename BufferType::iterator               IterType;
 
     /// @brief Initialize the stencil buffer with the values of voxel (i, j, k)
     /// and its neighbors.
@@ -68,7 +74,7 @@ public:
     {
         mCenter = ijk;
         mStencil[0] = mCache.getValue(ijk);
-        static_cast<StencilType&>(*this).init(mCenter);
+        static_cast<DerivedType&>(*this).init(mCenter);
     }
 
     /// @brief Initialize the stencil buffer with the values of voxel (i, j, k)
@@ -80,7 +86,7 @@ public:
     {
         mCenter = ijk;
         mStencil[0] = centerValue;
-        static_cast<StencilType&>(*this).init(mCenter);
+        static_cast<DerivedType&>(*this).init(mCenter);
     }
 
     /// @brief Initialize the stencil buffer with the values of voxel
@@ -93,7 +99,7 @@ public:
     {
         mCenter = iter.getCoord();
         mStencil[0] = *iter;
-        static_cast<StencilType&>(*this).init(mCenter);
+        static_cast<DerivedType&>(*this).init(mCenter);
     }
 
     /// @brief Initialize the stencil buffer with the values of voxel (x, y, z)
@@ -123,14 +129,14 @@ public:
     template<int i, int j, int k>
     inline const ValueType& getValue() const
     {
-        return mStencil[static_cast<const StencilType&>(*this).template pos<i,j,k>()];
+        return mStencil[static_cast<const DerivedType&>(*this).template pos<i,j,k>()];
     }
 
     /// @brief Set the value at the specified location relative to the center of the stencil
     template<int i, int j, int k>
     inline void setValue(const ValueType& value)
     {
-        mStencil[static_cast<const StencilType&>(*this).template pos<i,j,k>()] = value;
+        mStencil[static_cast<const DerivedType&>(*this).template pos<i,j,k>()] = value;
     }
 
     /// @brief Return the size of the stencil buffer.
@@ -139,7 +145,7 @@ public:
     /// @brief Return the median value of the current stencil.
     inline ValueType median() const
     {
-        std::vector<ValueType> tmp(mStencil);//local copy
+        BufferType tmp(mStencil);//local copy
         assert(!tmp.empty());
         size_t midpoint = (tmp.size() - 1) >> 1;
         // Partially sort the vector until the median value is at the midpoint.
@@ -198,9 +204,11 @@ public:
 
 protected:
     // Constructor is protected to prevent direct instantiation.
-    BaseStencil(const GridType& grid, int size):
-        mGrid(&grid), mCache(grid.getConstAccessor()),
-        mStencil(size), mCenter(Coord::max())
+    BaseStencil(const GridType& grid, int size)
+        : mGrid(&grid)
+        , mCache(grid.tree())
+        , mStencil(size)
+        , mCenter(Coord::max())
     {
     }
 
@@ -209,7 +217,7 @@ protected:
     BufferType      mStencil;
     Coord           mCenter;
 
-}; // class BaseStencil
+}; // BaseStencil class
 
 
 ////////////////////////////////////////
@@ -230,17 +238,19 @@ namespace { // anonymous namespace for stencil-layout map
 }
 
 
-template<typename GridType>
-class SevenPointStencil: public BaseStencil<GridType, SevenPointStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class SevenPointStencil: public BaseStencil<SevenPointStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef SevenPointStencil<GridT, IsSafe>  SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe> BaseType;
 public:
-    typedef BaseStencil<GridType, SevenPointStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType   BufferType;
-    typedef typename GridType::ValueType    ValueType;
-    typedef math::Vec3<ValueType>           Vec3Type;
+    typedef GridT                             GridType;
+    typedef typename GridT::TreeType          TreeType;
+    typedef typename GridT::ValueType         ValueType;
+    
     static const int SIZE = 7;
 
-    SevenPointStencil(const GridType& grid): BaseType(grid, SIZE) {}
+    SevenPointStencil(const GridT& grid): BaseType(grid, SIZE) {}
 
     /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
@@ -259,10 +269,10 @@ private:
         BaseType::template setValue< 0, 0, 1>(mCache.getValue(ijk.offsetBy( 0, 0, 1)));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
-};
+};// SevenPointStencil class
 
 
 ////////////////////////////////////////
@@ -282,14 +292,16 @@ namespace { // anonymous namespace for stencil-layout map
     template<> struct BoxPt< 1, 1, 0> { enum { idx = 7 }; };
 }
 
-template<typename GridType>
-class BoxStencil: public BaseStencil<GridType, BoxStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class BoxStencil: public BaseStencil<BoxStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef BoxStencil<GridT, IsSafe>         SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe> BaseType;
 public:
-    typedef BaseStencil<GridType, BoxStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType   BufferType;
-    typedef typename GridType::ValueType    ValueType;
-    typedef math::Vec3<ValueType>           Vec3Type;
+    typedef GridT                             GridType;
+    typedef typename GridT::TreeType          TreeType;
+    typedef typename GridT::ValueType         ValueType;
+    
     static const int SIZE = 8;
 
     BoxStencil(const GridType& grid): BaseType(grid, SIZE) {}
@@ -319,7 +331,7 @@ public:
     /// @note Trilinear interpolation kernal reads as:
     ///       v000 (1-u)(1-v)(1-w) + v001 (1-u)(1-v)w + v010 (1-u)v(1-w) + v011 (1-u)vw
     ///     + v100 u(1-v)(1-w)     + v101 u(1-v)w     + v110 uv(1-w)     + v111 uvw
-    inline ValueType interpolation(const Vec3Type& xyz) const
+    inline ValueType interpolation(const math::Vec3<ValueType>& xyz) const
     {
         const Real u = xyz[0] - BaseType::mCenter[0]; assert(u>=0 && u<=1);
         const Real v = xyz[1] - BaseType::mCenter[1]; assert(v>=0 && v<=1);
@@ -347,7 +359,7 @@ public:
     /// @note Computed as partial derivatives of the trilinear interpolation kernel:
     ///       v000 (1-u)(1-v)(1-w) + v001 (1-u)(1-v)w + v010 (1-u)v(1-w) + v011 (1-u)vw
     ///     + v100 u(1-v)(1-w)     + v101 u(1-v)w     + v110 uv(1-w)     + v111 uvw
-    inline Vec3Type gradient(const Vec3Type& xyz) const
+    inline math::Vec3<ValueType> gradient(const math::Vec3<ValueType>& xyz) const
     {
         const Real u = xyz[0] - BaseType::mCenter[0]; assert(u>=0 && u<=1);
         const Real v = xyz[1] - BaseType::mCenter[1]; assert(v>=0 && v<=1);
@@ -361,8 +373,9 @@ public:
         // Z component
         ValueType A = static_cast<ValueType>(D[0] + (D[1]- D[0]) * v);
         ValueType B = static_cast<ValueType>(D[2] + (D[3]- D[2]) * v);
-        Vec3Type grad(
-            zeroVal<ValueType>(), zeroVal<ValueType>(), static_cast<ValueType>(A + (B - A) * u));
+        math::Vec3<ValueType> grad(zeroVal<ValueType>(),
+                                   zeroVal<ValueType>(),
+                                   static_cast<ValueType>(A + (B - A) * u));
 
         D[0] = static_cast<ValueType>(BaseType::template getValue<0,0,0>() + D[0] * w);
         D[1] = static_cast<ValueType>(BaseType::template getValue<0,1,0>() + D[1] * w);
@@ -396,10 +409,10 @@ private:
         BaseType::template setValue< 1, 1, 0>(mCache.getValue(ijk.offsetBy( 1, 1, 0)));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
-};
+};// BoxStencil class
 
 
 ////////////////////////////////////////
@@ -438,14 +451,16 @@ namespace { // anonymous namespace for stencil-layout map
 }
 
 
-template<typename GridType>
-class SecondOrderDenseStencil: public BaseStencil<GridType, SecondOrderDenseStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class SecondOrderDenseStencil
+    : public BaseStencil<SecondOrderDenseStencil<GridT, IsSafe>, GridT, IsSafe >
 {
+    typedef SecondOrderDenseStencil<GridT, IsSafe> SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe >     BaseType;
 public:
-    typedef BaseStencil<GridType,SecondOrderDenseStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType   BufferType;
-    typedef typename GridType::ValueType    ValueType;
-    typedef math::Vec3<ValueType>           Vec3Type;
+    typedef GridT                                  GridType;
+    typedef typename GridT::TreeType               TreeType;
+    typedef typename GridType::ValueType           ValueType;
 
     static const int SIZE = 19;
 
@@ -482,10 +497,10 @@ private:
         mStencil[DensePt< 0, 1, 1>::idx] = mCache.getValue(ijk.offsetBy( 0,  1,  1));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
-};
+};// SecondOrderDenseStencil class
 
 
 ////////////////////////////////////////
@@ -516,14 +531,16 @@ namespace { // anonymous namespace for stencil-layout map
 }
 
 
-template<typename GridType>
-class ThirteenPointStencil: public BaseStencil<GridType, ThirteenPointStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class ThirteenPointStencil
+    : public BaseStencil<ThirteenPointStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef ThirteenPointStencil<GridT, IsSafe> SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe >  BaseType;
 public:
-    typedef BaseStencil<GridType, ThirteenPointStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType   BufferType;
-    typedef typename GridType::ValueType    ValueType;
-    typedef math::Vec3<ValueType>           Vec3Type;
+    typedef GridT                               GridType;
+    typedef typename GridT::TreeType            TreeType;
+    typedef typename GridType::ValueType        ValueType;
 
     static const int SIZE = 13;
 
@@ -552,10 +569,10 @@ private:
         mStencil[ThirteenPt< 0, 0,-2>::idx] = mCache.getValue(ijk.offsetBy( 0,  0, -2));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
-};
+};// ThirteenPointStencil class
 
 
 ////////////////////////////////////////
@@ -645,14 +662,16 @@ namespace { // anonymous namespace for stencil-layout map
 }
 
 
-template<typename GridType>
-class FourthOrderDenseStencil: public BaseStencil<GridType, FourthOrderDenseStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class FourthOrderDenseStencil
+    : public BaseStencil<FourthOrderDenseStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef FourthOrderDenseStencil<GridT, IsSafe> SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe >     BaseType;
 public:
-    typedef BaseStencil<GridType, FourthOrderDenseStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType   BufferType;
-    typedef typename GridType::ValueType    ValueType;
-    typedef math::Vec3<ValueType>           Vec3Type;
+    typedef GridT                                  GridType;
+    typedef typename GridT::TreeType               TreeType;
+    typedef typename GridType::ValueType           ValueType;
 
     static const int SIZE = 61;
 
@@ -740,10 +759,10 @@ private:
         mStencil[FourthDensePt< 0, 2,-2>::idx] = mCache.getValue(ijk.offsetBy( 0, 2,-2));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
-};
+};// FourthOrderDenseStencil class
 
 
 ////////////////////////////////////////
@@ -782,14 +801,16 @@ namespace { // anonymous namespace for stencil-layout map
 }
 
 
-template<typename GridType>
-class NineteenPointStencil: public BaseStencil<GridType, NineteenPointStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class NineteenPointStencil
+    : public BaseStencil<NineteenPointStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef NineteenPointStencil<GridT, IsSafe> SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe >  BaseType;
 public:
-    typedef BaseStencil<GridType, NineteenPointStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType   BufferType;
-    typedef typename GridType::ValueType    ValueType;
-    typedef math::Vec3<ValueType>           Vec3Type;
+    typedef GridT                               GridType;
+    typedef typename GridT::TreeType            TreeType;
+    typedef typename GridType::ValueType        ValueType;
 
     static const int SIZE = 19;
 
@@ -824,10 +845,10 @@ private:
         mStencil[NineteenPt< 0, 0,-3>::idx] = mCache.getValue(ijk.offsetBy( 0,  0, -3));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
-};
+};// NineteenPointStencil class
 
 
 ////////////////////////////////////////
@@ -996,14 +1017,16 @@ namespace { // anonymous namespace for stencil-layout map
 }
 
 
-template<typename GridType>
-class SixthOrderDenseStencil: public BaseStencil<GridType, SixthOrderDenseStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class SixthOrderDenseStencil
+    : public BaseStencil<SixthOrderDenseStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef SixthOrderDenseStencil<GridT, IsSafe> SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe >    BaseType;
 public:
-    typedef BaseStencil<GridType, SixthOrderDenseStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType   BufferType;
-    typedef typename GridType::ValueType    ValueType;
-    typedef math::Vec3<ValueType>           Vec3Type;
+    typedef GridT                                 GridType;
+    typedef typename GridT::TreeType              TreeType;
+    typedef typename GridType::ValueType          ValueType;
 
     static const int SIZE = 127;
 
@@ -1162,10 +1185,10 @@ private:
         mStencil[SixthDensePt< 0, 3,-3>::idx] = mCache.getValue(ijk.offsetBy( 0, 3,-3));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
-};
+};// SixthOrderDenseStencil class
 
 
 //////////////////////////////////////////////////////////////////////
@@ -1177,28 +1200,29 @@ private:
 ///
 /// @note For optimal random access performance this class
 /// includes its own grid accessor.
-template<typename GridType>
-class GradStencil: public BaseStencil<GridType, GradStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class GradStencil : public BaseStencil<GradStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef GradStencil<GridT, IsSafe>         SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe > BaseType;
 public:
-    typedef BaseStencil<GridType, GradStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType                 BufferType;
-    typedef typename GridType::ValueType                  ValueType;
-    typedef math::Vec3<ValueType>                         Vec3Type;
+    typedef GridT                              GridType;
+    typedef typename GridT::TreeType           TreeType;
+    typedef typename GridType::ValueType       ValueType;
 
     static const int SIZE = 7;
 
-    GradStencil(const GridType& grid):
-        BaseType(grid, SIZE),
-        mInv2Dx(ValueType(0.5 / grid.voxelSize()[0])),
-        mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
+    GradStencil(const GridType& grid)
+        : BaseType(grid, SIZE)
+        , mInv2Dx(ValueType(0.5 / grid.voxelSize()[0]))
+        , mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
     }
 
-    GradStencil(const GridType& grid, Real dx):
-        BaseType(grid, SIZE),
-        mInv2Dx(ValueType(0.5 / dx)),
-        mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
+    GradStencil(const GridType& grid, Real dx)
+        : BaseType(grid, SIZE)
+        , mInv2Dx(ValueType(0.5 / dx))
+        , mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
     }
 
@@ -1223,21 +1247,22 @@ public:
     ///
     /// @note This method should not be called until the stencil
     /// buffer has been populated via a call to moveTo(ijk).
-    inline Vec3Type gradient() const
+    inline math::Vec3<ValueType> gradient() const
     {
-        return Vec3Type(mStencil[2] - mStencil[1],
-                        mStencil[4] - mStencil[3],
-                        mStencil[6] - mStencil[5])*mInv2Dx;
+        return math::Vec3<ValueType>(mStencil[2] - mStencil[1],
+                                     mStencil[4] - mStencil[3],
+                                     mStencil[6] - mStencil[5])*mInv2Dx;
     }
     /// @brief Return the first-order upwind gradient corresponding to the direction V.
     ///
     /// @note This method should not be called until the stencil
     /// buffer has been populated via a call to moveTo(ijk).
-    inline Vec3Type gradient(const Vec3Type& V) const
+    inline math::Vec3<ValueType> gradient(const math::Vec3<ValueType>& V) const
     {
-        return Vec3Type(V[0]>0 ? mStencil[0] - mStencil[1] : mStencil[2] - mStencil[0],
-                        V[1]>0 ? mStencil[0] - mStencil[3] : mStencil[4] - mStencil[0],
-                        V[2]>0 ? mStencil[0] - mStencil[5] : mStencil[6] - mStencil[0])*2*mInv2Dx;
+        return math::Vec3<ValueType>(
+               V[0]>0 ? mStencil[0] - mStencil[1] : mStencil[2] - mStencil[0],
+               V[1]>0 ? mStencil[0] - mStencil[3] : mStencil[4] - mStencil[0],
+               V[2]>0 ? mStencil[0] - mStencil[5] : mStencil[6] - mStencil[0])*2*mInv2Dx;
     }
 
     /// Return the Laplacian computed at the previously buffered
@@ -1253,7 +1278,7 @@ public:
     /// is different from the signs of any of its six nearest neighbors.
     inline bool zeroCrossing() const
     {
-        const BufferType& v = mStencil;
+        const typename BaseType::BufferType& v = mStencil;
         return (v[0]>0 ? (v[1]<0 || v[2]<0 || v[3]<0 || v[4]<0 || v[5]<0 || v[6]<0)
                        : (v[1]>0 || v[2]>0 || v[3]>0 || v[4]>0 || v[5]>0 || v[6]>0));
     }
@@ -1265,13 +1290,13 @@ public:
     /// @note This method assumes that the grid represents a level set
     /// with distances in world units and a simple affine transfrom
     /// with uniform scaling.
-    inline Vec3Type cpt()
+    inline math::Vec3<ValueType> cpt()
     {
         const Coord& ijk = BaseType::getCenterCoord();
         const ValueType d = ValueType(mStencil[0] * 0.5 * mInvDx2); // distance in voxels / (2dx^2)
-        return Vec3Type(ijk[0] - d*(mStencil[2] - mStencil[1]),
-                        ijk[1] - d*(mStencil[4] - mStencil[3]),
-                        ijk[2] - d*(mStencil[6] - mStencil[5]));
+        return math::Vec3<ValueType>(ijk[0] - d*(mStencil[2] - mStencil[1]),
+                                     ijk[1] - d*(mStencil[4] - mStencil[3]),
+                                     ijk[2] - d*(mStencil[6] - mStencil[5]));
     }
 
 private:
@@ -1288,12 +1313,11 @@ private:
         mStencil[6] = mCache.getValue(ijk.offsetBy( 0,  0,  1));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
     const ValueType mInv2Dx, mInvDx2;
-}; // class GradStencil
-
+}; // GradStencil class
 
 ////////////////////////////////////////
 
@@ -1303,30 +1327,31 @@ private:
 ///
 /// @note For optimal random access performance this class
 /// includes its own grid accessor.
-template<typename GridType>
-class WenoStencil: public BaseStencil<GridType, WenoStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class WenoStencil: public BaseStencil<WenoStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef WenoStencil<GridT, IsSafe>         SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe > BaseType;
 public:
-    typedef BaseStencil<GridType, WenoStencil<GridType> > BaseType;
-    typedef typename BaseType::BufferType                 BufferType;
-    typedef typename GridType::ValueType                  ValueType;
-    typedef math::Vec3<ValueType>                         Vec3Type;
+    typedef GridT                              GridType;
+    typedef typename GridT::TreeType           TreeType;
+    typedef typename GridType::ValueType       ValueType;
 
     static const int SIZE = 19;
 
-    WenoStencil(const GridType& grid):
-        BaseType(grid, SIZE),
-        mDx2(ValueType(math::Pow2(grid.voxelSize()[0]))),
-        mInv2Dx(ValueType(0.5 / grid.voxelSize()[0])),
-        mInvDx2(ValueType(1.0 / mDx2))
+    WenoStencil(const GridType& grid)
+        : BaseType(grid, SIZE)
+        , mDx2(ValueType(math::Pow2(grid.voxelSize()[0])))
+        , mInv2Dx(ValueType(0.5 / grid.voxelSize()[0]))
+        , mInvDx2(ValueType(1.0 / mDx2))
     {
     }
 
-    WenoStencil(const GridType& grid, Real dx):
-        BaseType(grid, SIZE),
-        mDx2(ValueType(dx * dx)),
-        mInv2Dx(ValueType(0.5 / dx)),
-        mInvDx2(ValueType(1.0 / mDx2))
+    WenoStencil(const GridType& grid, Real dx)
+        : BaseType(grid, SIZE)
+        , mDx2(ValueType(dx * dx))
+        , mInv2Dx(ValueType(0.5 / dx))
+        , mInvDx2(ValueType(1.0 / mDx2))
     {
     }
 
@@ -1337,7 +1362,7 @@ public:
     /// buffer has been populated via a call to moveTo(ijk).
     inline ValueType normSqGrad() const
     {
-        const BufferType& v = mStencil;
+        const typename BaseType::BufferType& v = mStencil;
 #ifdef DWA_OPENVDB
         // SSE optimized
         const simd::Float4
@@ -1369,10 +1394,10 @@ public:
     ///
     /// @note This method should not be called until the stencil
     /// buffer has been populated via a call to moveTo(ijk).
-    inline Vec3Type gradient(const Vec3Type& V) const
+    inline math::Vec3<ValueType> gradient(const math::Vec3<ValueType>& V) const
     {
-        const BufferType& v = mStencil;
-        return 2*mInv2Dx * Vec3Type(
+        const typename BaseType::BufferType& v = mStencil;
+        return 2*mInv2Dx * math::Vec3<ValueType>(
             V[0]>0 ? math::WENO5(v[ 2]-v[ 1],v[ 3]-v[ 2],v[ 0]-v[ 3], v[ 4]-v[ 0],v[ 5]-v[ 4],mDx2)
                 : math::WENO5(v[ 6]-v[ 5],v[ 5]-v[ 4],v[ 4]-v[ 0], v[ 0]-v[ 3],v[ 3]-v[ 2],mDx2),
             V[1]>0 ? math::WENO5(v[ 8]-v[ 7],v[ 9]-v[ 8],v[ 0]-v[ 9], v[10]-v[ 0],v[11]-v[10],mDx2)
@@ -1385,12 +1410,11 @@ public:
     ///
     /// @note This method should not be called until the stencil
     /// buffer has been populated via a call to moveTo(ijk).
-    inline Vec3Type gradient() const
+    inline math::Vec3<ValueType> gradient() const
     {
-        return mInv2Dx * Vec3Type(
-            mStencil[ 4] - mStencil[ 3],
-            mStencil[10] - mStencil[ 9],
-            mStencil[16] - mStencil[15]);
+        return mInv2Dx * math::Vec3<ValueType>(mStencil[ 4] - mStencil[ 3],
+                                               mStencil[10] - mStencil[ 9],
+                                               mStencil[16] - mStencil[15]);
     }
 
     /// Return the Laplacian computed at the previously buffered
@@ -1410,7 +1434,7 @@ public:
     /// differs from the sign of any of its six nearest neighbors
     inline bool zeroCrossing() const
     {
-        const BufferType& v = mStencil;
+        const typename BaseType::BufferType& v = mStencil;
         return (v[ 0]>0 ? (v[ 3]<0 || v[ 4]<0 || v[ 9]<0 || v[10]<0 || v[15]<0 || v[16]<0)
                         : (v[ 3]>0 || v[ 4]>0 || v[ 9]>0 || v[10]>0 || v[15]>0 || v[16]>0));
     }
@@ -1440,36 +1464,39 @@ private:
         mStencil[18] = mCache.getValue(ijk.offsetBy( 0,  0,  3));
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
     const ValueType mDx2, mInv2Dx, mInvDx2;
-}; // class WenoStencil
+}; // WenoStencil class
 
 
 //////////////////////////////////////////////////////////////////////
 
 
-template<typename GridType>
-class CurvatureStencil: public BaseStencil<GridType, CurvatureStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class CurvatureStencil: public BaseStencil<CurvatureStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef CurvatureStencil<GridT, IsSafe>   SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe> BaseType;
 public:
-    typedef BaseStencil<GridType, CurvatureStencil<GridType> > BaseType;
-    typedef typename GridType::ValueType                       ValueType;
+    typedef GridT                             GridType;
+    typedef typename GridT::TreeType          TreeType;
+    typedef typename GridT::ValueType         ValueType;
 
      static const int SIZE = 19;
 
-    CurvatureStencil(const GridType& grid):
-        BaseType(grid, SIZE),
-        mInv2Dx(ValueType(0.5 / grid.voxelSize()[0])),
-        mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
+    CurvatureStencil(const GridType& grid)
+        : BaseType(grid, SIZE)
+        , mInv2Dx(ValueType(0.5 / grid.voxelSize()[0]))
+        , mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
     }
 
-    CurvatureStencil(const GridType& grid, Real dx):
-        BaseType(grid, SIZE),
-        mInv2Dx(ValueType(0.5 / dx)),
-        mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
+    CurvatureStencil(const GridType& grid, Real dx)
+        : BaseType(grid, SIZE)
+        , mInv2Dx(ValueType(0.5 / dx))
+        , mInvDx2(ValueType(4.0 * mInv2Dx * mInv2Dx))
     {
     }
 
@@ -1574,27 +1601,30 @@ private:
         return true;
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
     const ValueType mInv2Dx, mInvDx2;
-}; // class CurvatureStencil
+}; // CurvatureStencil class
 
 
 //////////////////////////////////////////////////////////////////////
 
 
 /// @brief Dense stencil of a given width
-template<typename GridType>
-class DenseStencil: public BaseStencil<GridType, DenseStencil<GridType> >
+template<typename GridT, bool IsSafe = true>
+class DenseStencil: public BaseStencil<DenseStencil<GridT, IsSafe>, GridT, IsSafe>
 {
+    typedef DenseStencil<GridT, IsSafe>       SelfT;
+    typedef BaseStencil<SelfT, GridT, IsSafe> BaseType;
 public:
-    typedef BaseStencil<GridType, DenseStencil<GridType> > BaseType;
-    typedef typename GridType::ValueType                   ValueType;
+    typedef GridT                             GridType;
+    typedef typename GridT::TreeType          TreeType;
+    typedef typename GridType::ValueType      ValueType;
 
-    DenseStencil(const GridType& grid, int halfWidth) :
-        BaseType(grid, /*size=*/math::Pow3(2 * halfWidth + 1)),
-        mHalfWidth(halfWidth)
+    DenseStencil(const GridType& grid, int halfWidth)
+        : BaseType(grid, /*size=*/math::Pow3(2 * halfWidth + 1))
+        , mHalfWidth(halfWidth)
     {
         assert(halfWidth>0);
     }
@@ -1632,11 +1662,11 @@ private:
         }
     }
 
-    template<typename, typename> friend class BaseStencil; // allow base class to call init()
+    template<typename, typename, bool> friend class BaseStencil; // allow base class to call init()
     using BaseType::mCache;
     using BaseType::mStencil;
     const int mHalfWidth;
-};
+};// DenseStencil class
 
 
 } // end math namespace
@@ -1645,6 +1675,6 @@ private:
 
 #endif // OPENVDB_MATH_STENCILS_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
