@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -347,6 +347,71 @@ Transform::postMult(const Mat3d& m)
 }
 
 
+////////////////////////////////////////
+
+
+BBoxd
+Transform::indexToWorld(const CoordBBox& indexBBox) const
+{
+    return this->indexToWorld(BBoxd(indexBBox.min().asVec3d(), indexBBox.max().asVec3d()));
+}
+
+
+BBoxd
+Transform::indexToWorld(const BBoxd& indexBBox) const
+{
+    const Vec3d &imin = indexBBox.min(), &imax = indexBBox.max();
+
+    Vec3d corners[8];
+    corners[0] = imin;
+    corners[1] = Vec3d(imax(0), imin(1), imin(2));
+    corners[2] = Vec3d(imax(0), imax(1), imin(2));
+    corners[3] = Vec3d(imin(0), imax(1), imin(2));
+    corners[4] = Vec3d(imin(0), imin(1), imax(2));
+    corners[5] = Vec3d(imax(0), imin(1), imax(2));
+    corners[6] = imax;
+    corners[7] = Vec3d(imin(0), imax(1), imax(2));
+
+    BBoxd worldBBox;
+    Vec3d &wmin = worldBBox.min(), &wmax = worldBBox.max();
+
+    wmin = wmax = this->indexToWorld(corners[0]);
+    for (int i = 1; i < 8; ++i) {
+        Vec3d image = this->indexToWorld(corners[i]);
+        wmin = minComponent(wmin, image);
+        wmax = maxComponent(wmax, image);
+    }
+    return worldBBox;
+}
+
+
+BBoxd
+Transform::worldToIndex(const BBoxd& worldBBox) const
+{
+    Vec3d indexMin, indexMax;
+    calculateBounds(*this, worldBBox.min(), worldBBox.max(), indexMin, indexMax);
+    return BBoxd(indexMin, indexMax);
+}
+
+
+CoordBBox
+Transform::worldToIndexCellCentered(const BBoxd& worldBBox) const
+{
+    Vec3d indexMin, indexMax;
+    calculateBounds(*this, worldBBox.min(), worldBBox.max(), indexMin, indexMax);
+    return CoordBBox(Coord::round(indexMin), Coord::round(indexMax));
+}
+
+
+CoordBBox
+Transform::worldToIndexNodeCentered(const BBoxd& worldBBox) const
+{
+    Vec3d indexMin, indexMax;
+    calculateBounds(*this, worldBBox.min(), worldBBox.max(), indexMin, indexMax);
+    return CoordBBox(Coord::floor(indexMin), Coord::floor(indexMax));
+}
+
+
 ////////////////////////////////////////
 
 // Utility methods
@@ -456,26 +521,27 @@ Transform::print(std::ostream& os, const std::string& indent) const
             linearRow.push_back(str);
         }
         w = std::max<size_t>(w, 30);
+        const int iw = int(w);
 
         // Print rows of the linear component matrix side-by-side with frustum parameters.
-        ostr << indent << std::left << std::setw(w) << "linear:"
+        ostr << indent << std::left << std::setw(iw) << "linear:"
             << "  frustum:\n";
-        ostr << indent << "   " << std::left << std::setw(w) << linearRow[0]
+        ostr << indent << "   " << std::left << std::setw(iw) << linearRow[0]
             << "  taper:  " << frustum.getTaper() << "\n";
-        ostr << indent << "   " << std::left << std::setw(w) << linearRow[1]
+        ostr << indent << "   " << std::left << std::setw(iw) << linearRow[1]
             << "  depth:  " << frustum.getDepth() << "\n";
 
         std::ostringstream ostmp;
-        ostmp << indent << "   " << std::left << std::setw(w) << linearRow[2]
+        ostmp << indent << "   " << std::left << std::setw(iw) << linearRow[2]
             << "  bounds: " << frustum.getBBox();
         if (ostmp.str().size() < 79) {
             ostr << ostmp.str() << "\n";
-            ostr << indent << "   " << std::left << std::setw(w) << linearRow[3] << "\n";
+            ostr << indent << "   " << std::left << std::setw(iw) << linearRow[3] << "\n";
         } else {
             // If the frustum bounding box doesn't fit on one line, split it into two lines.
-            ostr << indent << "   " << std::left << std::setw(w) << linearRow[2]
+            ostr << indent << "   " << std::left << std::setw(iw) << linearRow[2]
                 << "  bounds: " << frustum.getBBox().min() << " ->\n";
-            ostr << indent << "   " << std::left << std::setw(w) << linearRow[3]
+            ostr << indent << "   " << std::left << std::setw(iw) << linearRow[3]
                 << "             " << frustum.getBBox().max() << "\n";
         }
 
@@ -503,6 +569,6 @@ operator<<(std::ostream& os, const Transform& t)
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
