@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -46,9 +46,11 @@ namespace OPENVDB_VERSION_NAME {
 typedef tbb::mutex Mutex;
 typedef Mutex::scoped_lock Lock;
 
+namespace {
 // Declare this at file scope to ensure thread-safe initialization.
 Mutex sInitMutex;
 bool sIsInitialized = false;
+}
 
 void
 initialize()
@@ -103,7 +105,6 @@ initialize()
     DoubleGrid::registerGrid();
     Int32Grid::registerGrid();
     Int64Grid::registerGrid();
-    HermiteGrid::registerGrid();
     StringGrid::registerGrid();
     Vec3IGrid::registerGrid();
     Vec3SGrid::registerGrid();
@@ -167,6 +168,6 @@ __pragma(warning(default:1711))
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
