@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -39,94 +39,19 @@
 #ifndef OPENVDB_TOOLS_LEVEL_SET_ADVECT_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_LEVEL_SET_ADVECT_HAS_BEEN_INCLUDED
 
+#include <tbb/parallel_for.h>
+#include <tbb/parallel_reduce.h>
+#include <openvdb/Platform.h>
 #include "LevelSetTracker.h"
-#include "Interpolation.h" // for BoxSampler, etc.
+#include "VelocityFields.h" // for EnrightField
 #include <openvdb/math/FiniteDifference.h>
+#include <boost/math/constants/constants.hpp>
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace tools {
 
-/// Below are two simple wrapper classes for advection velocity fields
-/// DiscreteField wraps a velocity grid and EnrightField is mostly
-/// intended for debugging (it's an analytical divergence free and
-/// periodic field). They both share the same API required by the
-/// LevelSetAdvection class defined below. Thus, any class with this
-/// API should work with LevelSetAdvection.
-
-/// Note the Field wrapper classes below always assume the velocity
-/// is represented in the world-frame of reference. For DiscreteField
-/// this implies the input grid must contain velocities in world
-/// coordinates.
-
-/// @brief Thin wrapper class for a velocity grid
-/// @note Consider replacing BoxSampler with StaggeredBoxSampler
-template <typename VelGridT, typename Interpolator = BoxSampler>
-class DiscreteField
-{
-public:
-    typedef typename VelGridT::ValueType     VectorType;
-    typedef typename VectorType::ValueType   ScalarType;
-
-    DiscreteField(const VelGridT &vel): mAccessor(vel.tree()), mTransform(&vel.transform()) {}
-
-    /// @return const reference to the transfrom between world and index space
-    /// @note Use this method to determine if a client grid is
-    /// aligned with the coordinate space of the velocity grid.
-    const math::Transform& transform() const { return *mTransform; }
-
-    /// @return the interpolated velocity at the world space position xyz
-    inline VectorType operator() (const Vec3d& xyz, ScalarType) const
-    {
-        VectorType result = zeroVal<VectorType>();
-        Interpolator::sample(mAccessor, mTransform->worldToIndex(xyz), result);
-        return result;
-    }
-
-    /// @return the velocity at the coordinate space position ijk
-    inline VectorType operator() (const Coord& ijk, ScalarType) const
-    {
-        return mAccessor.getValue(ijk);
-    }
-
-private:
-    const typename VelGridT::ConstAccessor mAccessor;//Not thread-safe
-    const math::Transform*                 mTransform;
-
-}; // end of DiscreteField
-
-/// @brief Analytical, divergence-free and periodic vecloity field
-/// @note Primarily intended for debugging!
-/// @warning This analytical velocity only produce meaningfull values
-/// in the unitbox in world space. In other words make sure any level
-/// set surface in fully enclodes in the axis aligned bounding box
-/// spanning 0->1 in world units.
-template <typename ScalarT = float>
-class EnrightField
-{
-public:
-    typedef ScalarT             ScalarType;
-    typedef math::Vec3<ScalarT> VectorType;
-
-    EnrightField() {}
-
-    /// @return const reference to the identity transfrom between world and index space
-    /// @note Use this method to determine if a client grid is
-    /// aligned with the coordinate space of this velocity field
-    math::Transform transform() const { return math::Transform(); }
-
-    /// @return the velocity in world units, evaluated at the world
-    /// position xyz and at the specified time
-    inline VectorType operator() (const Vec3d& xyz, ScalarType time) const;
-
-    /// @return the velocity at the coordinate space position ijk
-    inline VectorType operator() (const Coord& ijk, ScalarType time) const
-    {
-        return (*this)(ijk.asVec3d(), time);
-    }
-}; // end of EnrightField
-
 /// @brief  Hyperbolic advection of narrow-band level sets in an
 /// external velocity field
 ///
@@ -138,7 +63,7 @@ public:
 /// class VelocityField {
 ///   ...
 /// public:
-///   openvdb::VectorType operator() (const openvdb::Coord& xyz, ScalarType time) const;
+///   openvdb::VectorType operator() (const openvdb::Coord& xyz, ValueType time) const;
 ///   ...
 /// };
 /// @endcode
@@ -175,10 +100,10 @@ class LevelSetAdvection
 public:
     typedef GridT                              GridType;
     typedef LevelSetTracker<GridT, InterruptT> TrackerT;
-    typedef typename TrackerT::RangeType       RangeType;
+    typedef typename TrackerT::LeafRange       LeafRange;
     typedef typename TrackerT::LeafType        LeafType;
     typedef typename TrackerT::BufferType      BufferType;
-    typedef typename TrackerT::ValueType       ScalarType;
+    typedef typename TrackerT::ValueType       ValueType;
     typedef typename FieldT::VectorType        VectorType;
 
     /// Main constructor
@@ -187,7 +112,7 @@ public:
         mSpatialScheme(math::HJWENO5_BIAS),
         mTemporalScheme(math::TVD_RK2) {}
 
-    virtual ~LevelSetAdvection() {};
+    virtual ~LevelSetAdvection() {}
 
     /// @return the spatial finite difference scheme
     math::BiasedGradientScheme getSpatialScheme() const { return mSpatialScheme; }
@@ -219,84 +144,90 @@ public:
     /// @return the grain-size used for multi-threading
     int  getGrainSize() const { return mTracker.getGrainSize(); }
     /// @brief Set the grain-size used for multi-threading.
-    /// @note A grainsize of 0 or less disables multi-threading!
+    /// @note A grain size of 0 or less disables multi-threading!
     void setGrainSize(int grainsize) { mTracker.setGrainSize(grainsize); }
 
-    /// Advect the level set from it's current time, time0, to it's
+    /// Advect the level set from its current time, time0, to its
     /// final time, time1. If time0>time1 backward advection is performed.
     ///
     /// @return number of CFL iterations used to advect from time0 to time1
-    size_t advect(ScalarType time0, ScalarType time1);
+    size_t advect(ValueType time0, ValueType time1);
 
 private:
 
-    // This templated private class implements all the level set magic.
+    // disallow copy construction and copy by assinment!
+    LevelSetAdvection(const LevelSetAdvection&);// not implemented
+    LevelSetAdvection& operator=(const LevelSetAdvection&);// not implemented
+
+    // This templated private struct implements all the level set magic.
     template<typename MapT, math::BiasedGradientScheme SpatialScheme,
              math::TemporalIntegrationScheme TemporalScheme>
-    class LevelSetAdvect
+    struct Advect
     {
-    public:
         /// Main constructor
-        LevelSetAdvect(LevelSetAdvection& parent);
+        Advect(LevelSetAdvection& parent);
         /// Shallow copy constructor called by tbb::parallel_for() threads
-        LevelSetAdvect(const LevelSetAdvect& other);
+        Advect(const Advect& other);
         /// Shallow copy constructor called by tbb::parallel_reduce() threads
-        LevelSetAdvect(LevelSetAdvect& other, tbb::split);
+        Advect(Advect& other, tbb::split);
         /// destructor
-        virtual ~LevelSetAdvect() {if (mIsMaster) this->clearField();};
-        /// Advect the level set from it's current time, time0, to it's final time, time1.
+        virtual ~Advect() { if (mIsMaster) this->clearField(); }
+        /// Advect the level set from its current time, time0, to its final time, time1.
         /// @return number of CFL iterations
-        size_t advect(ScalarType time0, ScalarType time1);
+        size_t advect(ValueType time0, ValueType time1);
         /// Used internally by tbb::parallel_for()
-        void operator()(const RangeType& r) const
+        void operator()(const LeafRange& r) const
         {
-            if (mTask) mTask(const_cast<LevelSetAdvect*>(this), r);
+            if (mTask) mTask(const_cast<Advect*>(this), r);
             else OPENVDB_THROW(ValueError, "task is undefined - don\'t call this method directly");
         }
         /// Used internally by tbb::parallel_reduce()
-        void operator()(const RangeType& r)
+        void operator()(const LeafRange& r)
         {
             if (mTask) mTask(this, r);
             else OPENVDB_THROW(ValueError, "task is undefined - don\'t call this method directly");
         }
         /// This is only called by tbb::parallel_reduce() threads
-        void join(const LevelSetAdvect& other) { mMaxAbsV = math::Max(mMaxAbsV, other.mMaxAbsV); }
-    private:
-        typedef typename boost::function<void (LevelSetAdvect*, const RangeType&)> FuncType;
-        LevelSetAdvection& mParent;
-        VectorType**       mVec;
-        const ScalarType   mMinAbsV;
-        ScalarType         mMaxAbsV;
-        const MapT*        mMap;
-        FuncType           mTask;
-        const bool         mIsMaster;
-        /// Enum to defeing the type of multi-threading
+        void join(const Advect& other) { mMaxAbsV = math::Max(mMaxAbsV, other.mMaxAbsV); }
+        /// Enum to define multi-threading type
         enum ThreadingMode { PARALLEL_FOR, PARALLEL_REDUCE }; // for internal use
         // method calling tbb
         void cook(ThreadingMode mode, size_t swapBuffer = 0);
         /// Sample field and return the CFT time step
-        typename GridT::ValueType sampleField(ScalarType time0, ScalarType time1);
+        typename GridT::ValueType sampleField(ValueType time0, ValueType time1);
         void  clearField();
-        void  sampleXformedField(const RangeType& r, ScalarType time0, ScalarType time1);
-        void  sampleAlignedField(const RangeType& r, ScalarType time0, ScalarType time1);
-        // Forward Euler advection steps: Phi(result) = Phi(0) - dt * V.Grad(0);
-        void euler1(const RangeType& r, ScalarType dt, Index resultBuffer);
+        void  sampleXformedField(const LeafRange& r, ValueType time0, ValueType time1);
+        void  sampleAlignedField(const LeafRange& r, ValueType time0, ValueType time1);
+
         // Convex combination of Phi and a forward Euler advection steps:
-        // Phi(result) = alpha * Phi(phi) + (1-alpha) * (Phi(0) - dt * V.Grad(0));
-        void euler2(const RangeType& r, ScalarType dt, ScalarType alpha, Index phiBuffer, Index resultBuffer);
-    }; // end of private LevelSetAdvect class
+        // Phi(result) = alpha * Phi(phi) + (1-alpha) * (Phi(0) - dt * Speed(speed)*|Grad[Phi(0)]|);
+        template <int Nominator, int Denominator>
+        void euler(const LeafRange&, ValueType, Index, Index);
+        inline void euler01(const LeafRange& r, ValueType t) {this->euler<0,1>(r, t, 0, 1);}
+        inline void euler12(const LeafRange& r, ValueType t) {this->euler<1,2>(r, t, 1, 1);}
+        inline void euler34(const LeafRange& r, ValueType t) {this->euler<3,4>(r, t, 1, 2);}
+        inline void euler13(const LeafRange& r, ValueType t) {this->euler<1,3>(r, t, 1, 2);}
+
+        LevelSetAdvection& mParent;
+        VectorType**       mVec;
+        const ValueType    mMinAbsV;
+        ValueType          mMaxAbsV;
+        const MapT*        mMap;
+        typename boost::function<void (Advect*, const LeafRange&)> mTask;
+        const bool         mIsMaster;
+    }; // end of private Advect struct
 
     template<math::BiasedGradientScheme SpatialScheme>
-    size_t advect1(ScalarType time0, ScalarType time1);
+    size_t advect1(ValueType time0, ValueType time1);
 
     template<math::BiasedGradientScheme SpatialScheme,
              math::TemporalIntegrationScheme TemporalScheme>
-    size_t advect2(ScalarType time0, ScalarType time1);
+    size_t advect2(ValueType time0, ValueType time1);
 
     template<math::BiasedGradientScheme SpatialScheme,
              math::TemporalIntegrationScheme TemporalScheme,
              typename MapType>
-    size_t advect3(ScalarType time0, ScalarType time1);
+    size_t advect3(ValueType time0, ValueType time1);
 
     TrackerT                        mTracker;
     //each thread needs a deep copy of the field since it might contain a ValueAccessor
@@ -304,14 +235,11 @@ private:
     math::BiasedGradientScheme      mSpatialScheme;
     math::TemporalIntegrationScheme mTemporalScheme;
 
-    // disallow copy by assignment
-    void operator=(const LevelSetAdvection& other) {}
-
 };//end of LevelSetAdvection
 
 template<typename GridT, typename FieldT, typename InterruptT>
 inline size_t
-LevelSetAdvection<GridT, FieldT, InterruptT>::advect(ScalarType time0, ScalarType time1)
+LevelSetAdvection<GridT, FieldT, InterruptT>::advect(ValueType time0, ValueType time1)
 {
     switch (mSpatialScheme) {
     case math::FIRST_BIAS:
@@ -333,7 +261,7 @@ LevelSetAdvection<GridT, FieldT, InterruptT>::advect(ScalarType time0, ScalarTyp
 template<typename GridT, typename FieldT, typename InterruptT>
 template<math::BiasedGradientScheme SpatialScheme>
 inline size_t
-LevelSetAdvection<GridT, FieldT, InterruptT>::advect1(ScalarType time0, ScalarType time1)
+LevelSetAdvection<GridT, FieldT, InterruptT>::advect1(ValueType time0, ValueType time1)
 {
     switch (mTemporalScheme) {
     case math::TVD_RK1:
@@ -352,7 +280,7 @@ template<typename GridT, typename FieldT, typename InterruptT>
 template<math::BiasedGradientScheme SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme>
 inline size_t
-LevelSetAdvection<GridT, FieldT, InterruptT>::advect2(ScalarType time0, ScalarType time1)
+LevelSetAdvection<GridT, FieldT, InterruptT>::advect2(ValueType time0, ValueType time1)
 {
     const math::Transform& trans = mTracker.grid().transform();
     if (trans.mapType() == math::UniformScaleMap::mapType()) {
@@ -374,30 +302,12 @@ template<math::BiasedGradientScheme SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme,
          typename MapT>
 inline size_t
-LevelSetAdvection<GridT, FieldT, InterruptT>::advect3(ScalarType time0, ScalarType time1)
+LevelSetAdvection<GridT, FieldT, InterruptT>::advect3(ValueType time0, ValueType time1)
 {
-    LevelSetAdvect<MapT, SpatialScheme, TemporalScheme> tmp(*this);
+    Advect<MapT, SpatialScheme, TemporalScheme> tmp(*this);
     return tmp.advect(time0, time1);
 }
 
-///////////////////////////////////////////////////////////////////////
-
-template <typename ScalarT>
-inline math::Vec3<ScalarT>
-EnrightField<ScalarT>::operator() (const Vec3d& xyz, ScalarType time) const
-{
-    static const ScalarT pi = ScalarT(3.1415926535897931), phase = pi / ScalarT(3.0);
-    const ScalarT Px =  pi * ScalarT(xyz[0]), Py = pi * ScalarT(xyz[1]), Pz = pi * ScalarT(xyz[2]);
-    const ScalarT tr =  cos(ScalarT(time) * phase);
-    const ScalarT a  =  sin(ScalarT(2.0)*Py);
-    const ScalarT b  = -sin(ScalarT(2.0)*Px);
-    const ScalarT c  =  sin(ScalarT(2.0)*Pz);
-    return math::Vec3<ScalarT>(
-        tr * ( ScalarT(2) * math::Pow2(sin(Px)) * a * c ),
-        tr * ( b * math::Pow2(sin(Py)) * c ),
-        tr * ( b * a * math::Pow2(sin(Pz)) ));
-}
-
 
 ///////////////////////////////////////////////////////////////////////
 
@@ -407,11 +317,11 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-LevelSetAdvect(LevelSetAdvection& parent):
+Advect<MapT, SpatialScheme, TemporalScheme>::
+Advect(LevelSetAdvection& parent):
     mParent(parent),
     mVec(NULL),
-    mMinAbsV(1e-6),
+    mMinAbsV(ValueType(1e-6)),
     mMap(parent.mTracker.grid().transform().template constMap<MapT>().get()),
     mTask(0),
     mIsMaster(true)
@@ -423,8 +333,8 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-LevelSetAdvect(const LevelSetAdvect& other):
+Advect<MapT, SpatialScheme, TemporalScheme>::
+Advect(const Advect& other):
     mParent(other.mParent),
     mVec(other.mVec),
     mMinAbsV(other.mMinAbsV),
@@ -440,8 +350,8 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-LevelSetAdvect(LevelSetAdvect& other, tbb::split):
+Advect<MapT, SpatialScheme, TemporalScheme>::
+Advect(Advect& other, tbb::split):
     mParent(other.mParent),
     mVec(other.mVec),
     mMinAbsV(other.mMinAbsV),
@@ -457,8 +367,8 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline size_t
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-advect(ScalarType time0, ScalarType time1)
+Advect<MapT, SpatialScheme, TemporalScheme>::
+advect(ValueType time0, ValueType time1)
 {
     size_t countCFL = 0;
     if ( math::isZero(time0 - time1) ) return countCFL;
@@ -467,7 +377,7 @@ advect(ScalarType time0, ScalarType time1)
         /// Make sure we have enough temporal auxiliary buffers
         mParent.mTracker.leafs().rebuildAuxBuffers(TemporalScheme == math::TVD_RK3 ? 2 : 1);
 
-        const ScalarType dt = this->sampleField(time0, time1);
+        const ValueType dt = this->sampleField(time0, time1);
         if ( math::isZero(dt) ) break;//V is essentially zero so terminate
 
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN //switch is resolved at compile-time
@@ -475,39 +385,45 @@ advect(ScalarType time0, ScalarType time1)
         case math::TVD_RK1:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(0)
-            mTask = boost::bind(&LevelSetAdvect::euler1, _1, _2, dt, /*result=*/1);
+            mTask = boost::bind(&Advect::euler01, _1, _2, dt);
+
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
             break;
         case math::TVD_RK2:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(0)
-            mTask = boost::bind(&LevelSetAdvect::euler1, _1, _2, dt, /*result=*/1);
+            mTask = boost::bind(&Advect::euler01, _1, _2, dt);
+
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt
             // Phi_t2(1) = 1/2 * Phi_t0(1) + 1/2 * (Phi_t1(0) - dt * V.Grad_t1(0))
-            mTask = boost::bind(&LevelSetAdvect::euler2, _1, _2, dt, ScalarType(0.5), /*phi=*/1, /*result=*/1);
+            mTask = boost::bind(&Advect::euler12, _1, _2, dt);
+
             // Cook and swap buffer 0 and 1 such that Phi_t2(0) and Phi_t1(1)
             this->cook(PARALLEL_FOR, 1);
             break;
         case math::TVD_RK3:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * VdotG_t0(0)
-            mTask = boost::bind(&LevelSetAdvect::euler1, _1, _2, dt, /*result=*/1);
+            mTask = boost::bind(&Advect::euler01, _1, _2, dt);
+
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt/2
             // Phi_t2(2) = 3/4 * Phi_t0(1) + 1/4 * (Phi_t1(0) - dt * V.Grad_t1(0))
-            mTask = boost::bind(&LevelSetAdvect::euler2, _1, _2, dt, ScalarType(0.75), /*phi=*/1, /*result=*/2);
+            mTask = boost::bind(&Advect::euler34, _1, _2, dt);
+
             // Cook and swap buffer 0 and 2 such that Phi_t2(0) and Phi_t1(2)
             this->cook(PARALLEL_FOR, 2);
 
             // Convex combine explict Euler step: t3 = t0 + dt
             // Phi_t3(2) = 1/3 * Phi_t0(1) + 2/3 * (Phi_t2(0) - dt * V.Grad_t2(0)
-            mTask = boost::bind(&LevelSetAdvect::euler2, _1, _2, dt, ScalarType(1.0/3.0), /*phi=*/1, /*result=*/2);
+            mTask = boost::bind(&Advect::euler13, _1, _2, dt);
+
             // Cook and swap buffer 0 and 2 such that Phi_t3(0) and Phi_t2(2)
             this->cook(PARALLEL_FOR, 2);
             break;
@@ -515,7 +431,7 @@ advect(ScalarType time0, ScalarType time1)
             OPENVDB_THROW(ValueError, "Temporal integration scheme not supported!");
         }//end of compile-time resolved switch
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_END
-            
+
         time0 += isForward ? dt : -dt;
         ++countCFL;
         mParent.mTracker.leafs().removeAuxBuffers();
@@ -531,25 +447,28 @@ template<typename MapT, math::BiasedGradientScheme SpatialScheme,
 	math::TemporalIntegrationScheme TemporalScheme>
 inline typename GridT::ValueType
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-sampleField(ScalarType time0, ScalarType time1)
+Advect<MapT, SpatialScheme, TemporalScheme>::
+sampleField(ValueType time0, ValueType time1)
 {
     mMaxAbsV = mMinAbsV;
     const size_t leafCount = mParent.mTracker.leafs().leafCount();
-    if (leafCount==0) return ScalarType(0.0);
+    if (leafCount==0) return ValueType(0.0);
     mVec = new VectorType*[leafCount];
     if (mParent.mField.transform() == mParent.mTracker.grid().transform()) {
-        mTask = boost::bind(&LevelSetAdvect::sampleAlignedField, _1, _2, time0, time1);
+        mTask = boost::bind(&Advect::sampleAlignedField, _1, _2, time0, time1);
     } else {
-        mTask = boost::bind(&LevelSetAdvect::sampleXformedField, _1, _2, time0, time1);
+        mTask = boost::bind(&Advect::sampleXformedField, _1, _2, time0, time1);
     }
     this->cook(PARALLEL_REDUCE);
-    if (math::isExactlyEqual(mMinAbsV, mMaxAbsV)) return ScalarType(0.0);//V is essentially zero
-    static const ScalarType CFL = (TemporalScheme == math::TVD_RK1 ? ScalarType(0.3) :
-                                   TemporalScheme == math::TVD_RK2 ? ScalarType(0.9) :
-                                   ScalarType(1.0))/math::Sqrt(ScalarType(3.0));
-    const ScalarType dt = math::Abs(time1 - time0), dx = mParent.mTracker.voxelSize();
-    return math::Min(dt, ScalarType(CFL*dx/math::Sqrt(mMaxAbsV)));
+    if (math::isExactlyEqual(mMinAbsV, mMaxAbsV)) return ValueType(0.0);//V is essentially zero
+#ifndef _MSC_VER // Visual C++ doesn't guarantee thread-safe initialization of local statics
+    static
+#endif
+    const ValueType CFL = (TemporalScheme == math::TVD_RK1 ? ValueType(0.3) :
+        TemporalScheme == math::TVD_RK2 ? ValueType(0.9) :
+        ValueType(1.0))/math::Sqrt(ValueType(3.0));
+    const ValueType dt = math::Abs(time1 - time0), dx = mParent.mTracker.voxelSize();
+    return math::Min(dt, ValueType(CFL*dx/math::Sqrt(mMaxAbsV)));
 }
 
 template<typename GridT, typename FieldT, typename InterruptT>
@@ -557,23 +476,21 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-sampleXformedField(const RangeType& range, ScalarType time0, ScalarType time1)
+Advect<MapT, SpatialScheme, TemporalScheme>::
+sampleXformedField(const LeafRange& range, ValueType time0, ValueType time1)
 {
     const bool isForward = time0 < time1;
     typedef typename LeafType::ValueOnCIter VoxelIterT;
     const MapT& map = *mMap;
     mParent.mTracker.checkInterrupter();
-    for (size_t n=range.begin(), e=range.end(); n != e; ++n) {
-        const LeafType& leaf = mParent.mTracker.leafs().leaf(n);
-        VectorType* vec = new VectorType[leaf.onVoxelCount()];
-        int m = 0;
-        for (VoxelIterT iter = leaf.cbeginValueOn(); iter; ++iter, ++m) {
-            const VectorType V = mParent.mField(map.applyMap(iter.getCoord().asVec3d()), time0);
-            mMaxAbsV = math::Max(mMaxAbsV, ScalarType(math::Pow2(V[0])+math::Pow2(V[1])+math::Pow2(V[2])));
-            vec[m] = isForward ? V : -V;
+    for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
+        VectorType* vec = new VectorType[leafIter->onVoxelCount()];
+        mVec[leafIter.pos()] = vec;
+        for (VoxelIterT iter = leafIter->cbeginValueOn(); iter; ++iter, ++vec) {
+            const VectorType v = mParent.mField(map.applyMap(iter.getCoord().asVec3d()), time0);
+            mMaxAbsV = math::Max(mMaxAbsV, ValueType(math::Pow2(v[0])+math::Pow2(v[1])+math::Pow2(v[2])));
+            *vec = isForward ? v : -v;
         }
-        mVec[n] = vec;
     }
 }
 
@@ -582,22 +499,20 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-sampleAlignedField(const RangeType& range, ScalarType time0, ScalarType time1)
+Advect<MapT, SpatialScheme, TemporalScheme>::
+sampleAlignedField(const LeafRange& range, ValueType time0, ValueType time1)
 {
     const bool isForward = time0 < time1;
     typedef typename LeafType::ValueOnCIter VoxelIterT;
     mParent.mTracker.checkInterrupter();
-    for (size_t n=range.begin(), e=range.end(); n != e; ++n) {
-        const LeafType& leaf = mParent.mTracker.leafs().leaf(n);
-        VectorType* vec = new VectorType[leaf.onVoxelCount()];
-        int m = 0;
-        for (VoxelIterT iter = leaf.cbeginValueOn(); iter; ++iter, ++m) {
-            const VectorType V = mParent.mField(iter.getCoord(), time0);
-            mMaxAbsV = math::Max(mMaxAbsV, ScalarType(math::Pow2(V[0])+math::Pow2(V[1])+math::Pow2(V[2])));
-            vec[m] = isForward ? V : -V;
+    for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
+        VectorType* vec = new VectorType[leafIter->onVoxelCount()];
+        mVec[leafIter.pos()] = vec;
+        for (VoxelIterT iter = leafIter->cbeginValueOn(); iter; ++iter, ++vec) {
+            const VectorType v = mParent.mField(iter.getCoord(), time0);
+            mMaxAbsV = math::Max(mMaxAbsV, ValueType(math::Pow2(v[0])+math::Pow2(v[1])+math::Pow2(v[2])));
+            *vec = isForward ? v : -v;
         }
-        mVec[n] = vec;
     }
 }
 
@@ -606,7 +521,7 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
+Advect<MapT, SpatialScheme, TemporalScheme>::
 clearField()
 {
     if (mVec == NULL) return;
@@ -620,84 +535,62 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
+Advect<MapT, SpatialScheme, TemporalScheme>::
 cook(ThreadingMode mode, size_t swapBuffer)
 {
     mParent.mTracker.startInterrupter("Advecting level set");
 
-    if (mParent.mTracker.getGrainSize()==0) {
-        (*this)(mParent.mTracker.leafs().getRange());
+    const int grainSize   = mParent.mTracker.getGrainSize();
+    const LeafRange range = mParent.mTracker.leafs().leafRange(grainSize);
+
+    if (grainSize == 0) {
+        (*this)(range);
     } else if (mode == PARALLEL_FOR) {
-        tbb::parallel_for(mParent.mTracker.leafs().getRange(mParent.mTracker.getGrainSize()), *this);
+        tbb::parallel_for(range, *this);
     } else if (mode == PARALLEL_REDUCE) {
-        tbb::parallel_reduce(mParent.mTracker.leafs().getRange(mParent.mTracker.getGrainSize()), *this);
+        tbb::parallel_reduce(range, *this);
     } else {
-        throw std::runtime_error("Undefined threading mode");
+        OPENVDB_THROW(ValueError,"Undefined threading mode");
     }
 
-    mParent.mTracker.leafs().swapLeafBuffer(swapBuffer, mParent.mTracker.getGrainSize()==0);
+    mParent.mTracker.leafs().swapLeafBuffer(swapBuffer, grainSize == 0);
 
     mParent.mTracker.endInterrupter();
 }
 
-// Forward Euler advection steps:
-// Phi(result) = Phi(0) - dt * V.Grad(0);
-template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
-inline void
-LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-euler1(const RangeType& range, ScalarType dt, Index resultBuffer)
-{
-    typedef math::BIAS_SCHEME<SpatialScheme>                             Scheme;
-    typedef typename Scheme::template ISStencil<GridType>::StencilType   Stencil;
-    typedef typename LeafType::ValueOnCIter VoxelIterT;
-    mParent.mTracker.checkInterrupter();
-    const MapT& map = *mMap;
-    typename TrackerT::LeafManagerType& leafs = mParent.mTracker.leafs();
-    Stencil stencil(mParent.mTracker.grid());
-    for (size_t n=range.begin(), e=range.end(); n != e; ++n) {
-        BufferType& result = leafs.getBuffer(n, resultBuffer);
-        const VectorType* vec = mVec[n];
-        int m=0;
-        for (VoxelIterT iter = leafs.leaf(n).cbeginValueOn(); iter; ++iter, ++m) {
-            stencil.moveTo(iter);
-            const VectorType V = vec[m], G = math::GradientBiased<MapT, SpatialScheme>::result(map, stencil, V);
-            result.setValue(iter.pos(), *iter - dt * V.dot(G));
-        }
-    }
-}
-
 // Convex combination of Phi and a forward Euler advection steps:
 // Phi(result) = alpha * Phi(phi) + (1-alpha) * (Phi(0) - dt * V.Grad(0));
 template<typename GridT, typename FieldT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
+template<typename MapT, math::BiasedGradientScheme SpatialScheme,
+         math::TemporalIntegrationScheme TemporalScheme>
+template <int Nominator, int Denominator>
 inline void
 LevelSetAdvection<GridT, FieldT, InterruptT>::
-LevelSetAdvect<MapT, SpatialScheme, TemporalScheme>::
-euler2(const RangeType& range, ScalarType dt, ScalarType alpha, Index phiBuffer, Index resultBuffer)
+Advect<MapT, SpatialScheme, TemporalScheme>::
+euler(const LeafRange& range, ValueType dt, Index phiBuffer, Index resultBuffer)
 {
-    typedef math::BIAS_SCHEME<SpatialScheme>                             Scheme;
-    typedef typename Scheme::template ISStencil<GridType>::StencilType   Stencil;
-    typedef typename LeafType::ValueOnCIter VoxelIterT;
+    typedef math::BIAS_SCHEME<SpatialScheme>                             SchemeT;
+    typedef typename SchemeT::template ISStencil<GridType>::StencilType  StencilT;
+    typedef typename LeafType::ValueOnCIter                              VoxelIterT;
+    typedef math::GradientBiased<MapT, SpatialScheme>                    GradT;
+
+    static const ValueType Alpha = ValueType(Nominator)/ValueType(Denominator);
+    static const ValueType Beta  = ValueType(1) - Alpha;
+
     mParent.mTracker.checkInterrupter();
     const MapT& map = *mMap;
-    typename TrackerT::LeafManagerType& leafs = mParent.mTracker.leafs();
-    const ScalarType beta = ScalarType(1.0) - alpha;
-    Stencil stencil(mParent.mTracker.grid());
-    for (size_t n=range.begin(), e=range.end(); n != e; ++n) {
-        const BufferType& phi = leafs.getBuffer(n, phiBuffer);
-        BufferType& result = leafs.getBuffer(n, resultBuffer);
-        const VectorType* vec = mVec[n];
-        int m=0;
-        for (VoxelIterT iter = leafs.leaf(n).cbeginValueOn(); iter; ++iter, ++m) {
-            stencil.moveTo(iter);
-            const VectorType V = vec[m], G = math::GradientBiased<MapT, SpatialScheme>::result(map, stencil, V);
-            result.setValue(iter.pos(), alpha*phi[iter.pos()] + beta*(*iter - dt * V.dot(G)));
-        }
-    }
+    StencilT stencil(mParent.mTracker.grid());
+    for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
+        const VectorType* v = mVec[leafIter.pos()];
+        const ValueType* phi = leafIter.buffer(phiBuffer).data();
+        ValueType* result = leafIter.buffer(resultBuffer).data();
+        for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter, ++v) {
+            const Index i = voxelIter.pos();
+            stencil.moveTo(voxelIter);
+            const ValueType a = stencil.getValue() - dt * v->dot(GradT::result(map, stencil,*v));
+            result[i] = Nominator ? Alpha * phi[i] + Beta * a : a;
+        }//loop over active voxels in the leaf of the mask
+    }//loop over leafs of the level set
 }
 
 } // namespace tools
@@ -706,6 +599,6 @@ euler2(const RangeType& range, ScalarType dt, ScalarType alpha, Index phiBuffer,
 
 #endif // OPENVDB_TOOLS_LEVEL_SET_ADVECT_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
