@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -268,10 +268,15 @@ doLevelSetRebuild(const GridType& grid, typename GridType::ValueType iso,
         primCpy.runParallel();
     }
 
-    MeshToVolume<GridType, InterruptT> vol(transform, OUTPUT_RAW_DATA, interrupter);
-    vol.convertToLevelSet(points, primitives, exBandWidth, inBandWidth);
+    QuadAndTriangleDataAdapter<Vec3s, Vec4I> mesh(points, primitives);
 
-    return vol.distGridPtr();
+    if (interrupter) {
+        return meshToVolume<GridType>(*interrupter, mesh, *transform, exBandWidth, inBandWidth,
+            DISABLE_RENORMALIZATION, NULL);
+    }
+
+    return meshToVolume<GridType>(mesh, *transform, exBandWidth, inBandWidth,
+        DISABLE_RENORMALIZATION, NULL);
 }
 
 
@@ -299,9 +304,9 @@ levelSetRebuild(const GridType& grid, float iso, float exWidth, float inWidth,
 {
     typedef typename GridType::ValueType ValueT;
     ValueT
-        isovalue(zeroVal<ValueT>() + iso),
-        exBandWidth(zeroVal<ValueT>() + exWidth),
-        inBandWidth(zeroVal<ValueT>() + inWidth);
+        isovalue(zeroVal<ValueT>() + ValueT(iso)),
+        exBandWidth(zeroVal<ValueT>() + ValueT(exWidth)),
+        inBandWidth(zeroVal<ValueT>() + ValueT(inWidth));
 
     return doLevelSetRebuild(grid, isovalue, exBandWidth, inBandWidth, xform, interrupter);
 }
@@ -314,9 +319,9 @@ levelSetRebuild(const GridType& grid, float iso, float exWidth, float inWidth,
 {
     typedef typename GridType::ValueType ValueT;
     ValueT
-        isovalue(zeroVal<ValueT>() + iso),
-        exBandWidth(zeroVal<ValueT>() + exWidth),
-        inBandWidth(zeroVal<ValueT>() + inWidth);
+        isovalue(zeroVal<ValueT>() + ValueT(iso)),
+        exBandWidth(zeroVal<ValueT>() + ValueT(exWidth)),
+        inBandWidth(zeroVal<ValueT>() + ValueT(inWidth));
 
     return doLevelSetRebuild<GridType, util::NullInterrupter>(
         grid, isovalue, exBandWidth, inBandWidth, xform, NULL);
@@ -329,8 +334,8 @@ levelSetRebuild(const GridType& grid, float iso, float halfVal, const math::Tran
 {
     typedef typename GridType::ValueType ValueT;
     ValueT
-        isovalue(zeroVal<ValueT>() + iso),
-        halfWidth(zeroVal<ValueT>() + halfVal);
+        isovalue(zeroVal<ValueT>() + ValueT(iso)),
+        halfWidth(zeroVal<ValueT>() + ValueT(halfVal));
 
     return doLevelSetRebuild<GridType, util::NullInterrupter>(
         grid, isovalue, halfWidth, halfWidth, xform, NULL);
@@ -343,6 +348,6 @@ levelSetRebuild(const GridType& grid, float iso, float halfVal, const math::Tran
 
 #endif // OPENVDB_TOOLS_LEVELSETREBUILD_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
