@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -137,7 +137,6 @@ TestGradient::testISGradientStencil()
 {
     using namespace openvdb;
 
-    typedef FloatGrid::ConstAccessor  AccessorType;
     FloatGrid::Ptr grid = createGrid<FloatGrid>(/*background=*/5.0);
     FloatTree& tree = grid->tree();
 
@@ -190,7 +189,6 @@ TestGradient::testISGradientStencil()
 
     result = math::ISGradient<math::BD_WENO5>::result(stencil);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, result.length(), /*tolerance=*/0.01);
-
 }
 
 
@@ -320,9 +318,9 @@ void
 TestGradient::testWSGradientStencilFrustum()
 {
     using namespace openvdb;
-    
+
     // Construct a frustum that matches the one in TestMaps::testFrustum()
-    
+
     openvdb::BBoxd bbox(Vec3d(0), Vec3d(100));
     math::NonlinearFrustumMap frustum(bbox, 1./6., 5);
     /// frustum will have depth, far plane - near plane = 5
@@ -333,9 +331,9 @@ TestGradient::testWSGradientStencilFrustum()
         boost::static_pointer_cast<math::NonlinearFrustumMap, math::MapBase>(
             frustum.preScale(Vec3d(10,10,10))->postTranslate(trans));
 
-    
+
     // Create a grid with this frustum
-    
+
     FloatGrid::Ptr grid = FloatGrid::create(/*background=*/0.f);
     math::Transform::Ptr transform = math::Transform::Ptr( new math::Transform(map));
     grid->setTransform(transform);
@@ -344,10 +342,10 @@ TestGradient::testWSGradientStencilFrustum()
     // Totally fill the interior of the frustum with word space distances
     // from its center.
 
-    
+
     math::Vec3d isCenter(.5 * 101, .5 * 101, .5 * 101);
     math::Vec3d wsCenter = map->applyMap(isCenter);
-    
+
     math::Coord ijk;
 
     // convert to IntType
@@ -358,19 +356,19 @@ TestGradient::testWSGradientStencilFrustum()
         for (ijk[1] = min.y(); ijk[1] < max.y(); ++ijk[1]) {
             for (ijk[2] = min.z(); ijk[2] < max.z(); ++ijk[2]) {
                 const math::Vec3d wsLocation = transform->indexToWorld(ijk);
-                const float dis = (wsLocation - wsCenter).length();
-                
+                const float dis = float((wsLocation - wsCenter).length());
+
                 acc.setValue(ijk, dis);
             }
         }
     }
-    
+
 
     {
     // test at location 10, 10, 10 in index space
     math::Coord xyz(10, 10, 10);
-    
-    math::Vec3s result = 
+
+    math::Vec3s result =
           math::Gradient<math::NonlinearFrustumMap, math::CD_2ND>::result(*map, acc, xyz);
 
     // The Gradient should be unit lenght for this case
@@ -379,16 +377,16 @@ TestGradient::testWSGradientStencilFrustum()
     math::Vec3d wsVec = transform->indexToWorld(xyz);
     math::Vec3d direction = (wsVec - wsCenter);
     direction.normalize();
-     
+
     // test the actual direction of the gradient
     CPPUNIT_ASSERT(direction.eq(result, 0.01 /*tolerance*/));
-    }              
+    }
 
     {
     // test at location 30, 30, 60 in index space
     math::Coord xyz(30, 30, 60);
-    
-    math::Vec3s result = 
+
+    math::Vec3s result =
           math::Gradient<math::NonlinearFrustumMap, math::CD_2ND>::result(*map, acc, xyz);
 
     // The Gradient should be unit lenght for this case
@@ -397,12 +395,12 @@ TestGradient::testWSGradientStencilFrustum()
     math::Vec3d wsVec = transform->indexToWorld(xyz);
     math::Vec3d direction = (wsVec - wsCenter);
     direction.normalize();
-     
+
     // test the actual direction of the gradient
     CPPUNIT_ASSERT(direction.eq(result, 0.01 /*tolerance*/));
-    }              
+    }
 }
-    
+
 
 
 void
@@ -410,8 +408,6 @@ TestGradient::testWSGradientStencil()
 {
     using namespace openvdb;
 
-    typedef FloatGrid::ConstAccessor  AccessorType;
-
     double voxel_size = 0.5;
     FloatGrid::Ptr grid = FloatGrid::create(/*background=*/5.0);
     grid->setTransform(math::Transform::createLinearTransform(voxel_size));
@@ -458,7 +454,6 @@ TestGradient::testWSGradientStencil()
             *affine_map, dense_4thOrder);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, result.length(), /*tolerance=*/0.01);
     }
-
     {
         math::UniformScaleTranslateMap map(voxel_size, Vec3d(0,0,0));
 
@@ -470,14 +465,12 @@ TestGradient::testWSGradientStencil()
         result = math::Gradient<math::ScaleTranslateMap, math::CD_2ND>::result(map, stencil);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, result.length(), /*tolerance=*/0.01);
     }
-
     {
         math::TranslationMap map;
         result = math::Gradient<math::TranslationMap, math::CD_2ND>::result(map, stencil);
         // value = 1 because the translation map assumes uniform spacing
         CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, result.length(), /*tolerance=*/0.01);
     }
-
     {
         // test the GenericMap Grid interface
         math::GenericMap generic_map(*grid);
@@ -584,7 +577,6 @@ TestGradient::testWSGradientNormSqrStencil()
 {
     using namespace openvdb;
 
-    typedef FloatGrid::ConstAccessor  AccessorType;
     double voxel_size = 0.5;
     FloatGrid::Ptr grid = FloatGrid::create(/*background=*/5.0);
     grid->setTransform(math::Transform::createLinearTransform(voxel_size));
@@ -640,7 +632,6 @@ TestGradient::testGradientTool()
 {
     using namespace openvdb;
 
-    typedef FloatGrid::ConstAccessor  AccessorType;
     FloatGrid::Ptr grid = createGrid<FloatGrid>(/*background=*/5.0);
     FloatTree& tree = grid->tree();
 
@@ -653,7 +644,6 @@ TestGradient::testGradientTool()
     CPPUNIT_ASSERT_EQUAL(dim[0]*dim[1]*dim[2], int(tree.activeVoxelCount()));
     const Coord xyz(10, 20, 30);
 
-
     Vec3SGrid::Ptr grad = tools::gradient(*grid);
     CPPUNIT_ASSERT_EQUAL(int(tree.activeVoxelCount()), int(grad->activeVoxelCount()));
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, grad->getConstAccessor().getValue(xyz).length(),
@@ -666,7 +656,6 @@ TestGradient::testGradientMaskedTool()
 {
     using namespace openvdb;
 
-    typedef FloatGrid::ConstAccessor  AccessorType;
     FloatGrid::Ptr grid = createGrid<FloatGrid>(/*background=*/5.0);
     FloatTree& tree = grid->tree();
 
@@ -677,18 +666,18 @@ TestGradient::testGradientMaskedTool()
 
     CPPUNIT_ASSERT(!tree.empty());
     CPPUNIT_ASSERT_EQUAL(dim[0]*dim[1]*dim[2], int(tree.activeVoxelCount()));
-  
+
     const openvdb::CoordBBox maskbbox(openvdb::Coord(35, 30, 30), openvdb::Coord(41, 41, 41));
     BoolGrid::Ptr maskGrid = BoolGrid::create(false);
     maskGrid->fill(maskbbox, true/*value*/, true/*activate*/);
 
     Vec3SGrid::Ptr grad = tools::gradient(*grid, *maskGrid);
     {// outside the masked region
-        const Coord xyz(10, 20, 30); 
+        const Coord xyz(10, 20, 30);
         CPPUNIT_ASSERT(!maskbbox.isInside(xyz));
         CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, grad->getConstAccessor().getValue(xyz).length(),
                                      /*tolerance=*/0.01);
-    } 
+    }
     {// inside the masked region
         const Coord xyz(38, 35, 33);
         CPPUNIT_ASSERT(maskbbox.isInside(xyz));
@@ -826,6 +815,6 @@ TestGradient::testOldStyleStencils()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, cs.gradient().length(), /*tolerance=*/0.01);
 }
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
