@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -402,7 +402,7 @@ TestLevelSetRayIntersector::stats()
         }
         timer.stop();
 
-        film.savePPM("/tmp/sphere_serial");
+        film.savePPM("sphere_serial");
         stats.print("First hit");
         hist.print("First hit");
     }
@@ -411,6 +411,6 @@ TestLevelSetRayIntersector::stats()
 
 #undef STATS_TEST
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
