@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -45,6 +45,7 @@ public:
     CPPUNIT_TEST(testCopyConstructor);
     CPPUNIT_TEST(testCopyConstructorEmpty);
     CPPUNIT_TEST(testAssignment);
+    CPPUNIT_TEST(testEquality);
     CPPUNIT_TEST_SUITE_END();
 
     void testInsert();
@@ -55,6 +56,7 @@ public:
     void testCopyConstructor();
     void testCopyConstructorEmpty();
     void testAssignment();
+    void testEquality();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestMetaMap);
@@ -128,7 +130,7 @@ TestMetaMap::testRemove()
 
     meta.removeMeta("meta3");
 
-    CPPUNIT_ASSERT(meta.empty());
+    CPPUNIT_ASSERT_EQUAL(0, int(meta.metaCount()));
 }
 
 void
@@ -153,7 +155,7 @@ TestMetaMap::testGetMetadata()
     //CPPUNIT_ASSERT(dm->value() == 2.0);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0,cdm->value(),0);
 
-    CPPUNIT_ASSERT(meta.getMetadata<StringMetadata>("meta2") == NULL);
+    CPPUNIT_ASSERT(!meta.getMetadata<StringMetadata>("meta2"));
 
     CPPUNIT_ASSERT_THROW(meta.metaValue<int32_t>("meta3"),
                          openvdb::TypeError);
@@ -182,7 +184,7 @@ TestMetaMap::testIO()
     MetaMap meta2;
     std::istringstream istr(ostr.str(), std::ios_base::binary);
     CPPUNIT_ASSERT_NO_THROW(meta2.readMeta(istr));
-    CPPUNIT_ASSERT(meta2.empty());
+    CPPUNIT_ASSERT_EQUAL(0, int(meta2.metaCount()));
 
     // Register just one of the three types, then reread and verify that
     // the value of the registered type can be retrieved.
@@ -293,7 +295,7 @@ TestMetaMap::testAssignment()
 
     // Create an empty map.
     MetaMap meta2;
-    CPPUNIT_ASSERT(meta2.empty());
+    CPPUNIT_ASSERT_EQUAL(0, int(meta2.metaCount()));
 
     // Copy the first map to the second.
     meta2 = meta;
@@ -312,6 +314,57 @@ TestMetaMap::testAssignment()
     CPPUNIT_ASSERT_EQUAL(std::string("testing"), meta2.metaValue<std::string>("meta1"));
 }
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+
+void
+TestMetaMap::testEquality()
+{
+    using namespace openvdb;
+
+    // Populate a map with data.
+    MetaMap meta;
+    meta.insertMeta("meta1", StringMetadata("testing"));
+    meta.insertMeta("meta2", Int32Metadata(20));
+    meta.insertMeta("meta3", FloatMetadata(3.14159f));
+
+    // Create an empty map.
+    MetaMap meta2;
+
+    // Verify that the two maps differ.
+    CPPUNIT_ASSERT(meta != meta2);
+    CPPUNIT_ASSERT(meta2 != meta);
+
+    // Copy the first map to the second.
+    meta2 = meta;
+
+    // Verify that the two maps are equivalent.
+    CPPUNIT_ASSERT(meta == meta2);
+    CPPUNIT_ASSERT(meta2 == meta);
+
+    // Modify the first map.
+    meta.removeMeta("meta1");
+    meta.insertMeta("abc", DoubleMetadata(2.0));
+
+    // Verify that the two maps differ.
+    CPPUNIT_ASSERT(meta != meta2);
+    CPPUNIT_ASSERT(meta2 != meta);
+
+    // Modify the second map and verify that the two maps differ.
+    meta2 = meta;
+    meta2.insertMeta("meta2", Int32Metadata(42));
+    CPPUNIT_ASSERT(meta != meta2);
+    CPPUNIT_ASSERT(meta2 != meta);
+
+    meta2 = meta;
+    meta2.insertMeta("meta3", FloatMetadata(2.0001f));
+    CPPUNIT_ASSERT(meta != meta2);
+    CPPUNIT_ASSERT(meta2 != meta);
+
+    meta2 = meta;
+    meta2.insertMeta("abc", DoubleMetadata(2.0001));
+    CPPUNIT_ASSERT(meta != meta2);
+    CPPUNIT_ASSERT(meta2 != meta);
+}
+
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
