@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -72,6 +72,7 @@ TestStats::testExtrema()
         CPPUNIT_ASSERT_EQUAL(2, int(s.size()));
         CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, s.min(), 0.000001);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, s.max(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, s.range(), 0.000001);
         //s.print("test");
     }
     {// non-trivial test
@@ -81,6 +82,7 @@ TestStats::testExtrema()
         CPPUNIT_ASSERT_EQUAL(5, int(s.size()));
         CPPUNIT_ASSERT_DOUBLES_EQUAL(data[2], s.min(), 0.000001);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(data[0], s.max(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(data[0]-data[2], s.range(), 0.000001);
         //s.print("test");
     }
     {// non-trivial test of Extrema::add(Extrema)
@@ -92,6 +94,7 @@ TestStats::testExtrema()
         CPPUNIT_ASSERT_EQUAL(5, int(s.size()));
         CPPUNIT_ASSERT_DOUBLES_EQUAL(data[2], s.min(), 0.000001);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(data[0], s.max(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(data[0]-data[2], s.range(), 0.000001);
         //s.print("test");
     }
     {// Trivial test of Extrema::add(value, n)
@@ -102,6 +105,7 @@ TestStats::testExtrema()
         CPPUNIT_ASSERT_EQUAL(n, s.size());
         CPPUNIT_ASSERT_DOUBLES_EQUAL(val, s.min(), 0.000001);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(val, s.max(), 0.000001);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, s.range(), 0.000001);
     }
     {// Test 1 of Extrema::add(value), Extrema::add(value, n) and Extrema::add(Extrema)
         openvdb::math::Extrema s, t;
@@ -713,6 +717,6 @@ TestStats::testGridHistogram()
     }
 }
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
