@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -35,6 +35,7 @@
 #include <openvdb/Types.h>
 #include <openvdb/openvdb.h>
 #include <openvdb/tools/ChangeBackground.h>
+#include <openvdb/tools/Diagnostics.h>
 #include <openvdb/tools/Clip.h>
 #include <openvdb/tools/GridOperators.h>
 #include <openvdb/tools/Filter.h>
@@ -46,8 +47,10 @@
 #include <openvdb/tools/Morphology.h>
 #include <openvdb/tools/PointAdvect.h>
 #include <openvdb/tools/PointScatter.h>
+#include <openvdb/tools/Prune.h>
 #include <openvdb/tools/ValueTransformer.h>
 #include <openvdb/tools/VectorTransformer.h>
+#include <openvdb/tools/VolumeAdvect.h>
 #include <openvdb/util/Util.h>
 #include <openvdb/util/CpuTimer.h>
 #include <openvdb/math/Stats.h>
@@ -78,6 +81,8 @@ public:
     CPPUNIT_TEST(testMaskedNormalize);
     CPPUNIT_TEST(testPointAdvect);
     CPPUNIT_TEST(testPointScatter);
+    CPPUNIT_TEST(testPrune);
+    CPPUNIT_TEST(testVolumeAdvect);
     CPPUNIT_TEST(testTransformValues);
     CPPUNIT_TEST(testVectorApply);
     CPPUNIT_TEST(testAccumulate);
@@ -102,6 +107,8 @@ public:
     void testMaskedNormalize();
     void testPointAdvect();
     void testPointScatter();
+    void testPrune();
+    void testVolumeAdvect();
     void testTransformValues();
     void testVectorApply();
     void testAccumulate();
@@ -128,7 +135,7 @@ public:
     void operator()(const std::string& name, float time, size_t n)
     {
         std::ostringstream ostr;
-        ostr << "/tmp/" << name << "_" << mVersion << "_" << mFrame << ".vdb";
+        ostr << name << "_" << mVersion << "_" << mFrame << ".vdb";
         openvdb::io::File file(ostr.str());
         openvdb::GridPtrVec grids;
         grids.push_back(mGrid);
@@ -620,40 +627,49 @@ TestTools::testErodeVoxels()
             }
         }
     }
-    /* Not completed yet 
+
+#if 0
     {//erode18
-        //for (int x=0; x<8; ++x) {
-        //for (int y=0; y<8; ++y) {
-        //for (int z=0; z<8; ++z) {
-        int x=3, y=3, z=3;
-                    const openvdb::Coord ijk(x,y,z);
-                    std::cerr << ijk;
-                    tree->clear();
-                    CPPUNIT_ASSERT_EQUAL(Index64(0), tree->activeVoxelCount());
-                    tree->setValue(ijk, 1.0f);
-                    CPPUNIT_ASSERT_EQUAL(Index64(1), tree->activeVoxelCount());
-                    CPPUNIT_ASSERT(tree->isValueOn(ijk));
-                    openvdb::tools::dilateVoxels(*tree, 1, openvdb::tools::NN_FACE_EDGE);
-                    std::cerr << " dilated to " << tree->activeVoxelCount();
-                    CPPUNIT_ASSERT_EQUAL(Index64(1 + 6 + 12), tree->activeVoxelCount());
-                    openvdb::tools::erodeVoxels( *tree, 1, openvdb::tools::NN_FACE_EDGE);
-                    std::cerr << ", eroded to " << tree->activeVoxelCount() << std::endl;
-                    CPPUNIT_ASSERT_EQUAL(Index64(1), tree->activeVoxelCount());
-                    CPPUNIT_ASSERT(tree->isValueOn(ijk));
-                    //      }
-                    //}
-                    //}
+        /// @todo Not implemented yet
+        for (int iter=1; iter<4; ++iter) {
+            for (int x=0; x<8; ++x) {
+                for (int y=0; y<8; ++y) {
+                    for (int z=0; z<8; ++z) {
+                        const openvdb::Coord ijk(x,y,z);
+                        tree->clear();
+                        CPPUNIT_ASSERT_EQUAL(Index64(0), tree->activeVoxelCount());
+                        tree->setValue(ijk, 1.0f);
+                        CPPUNIT_ASSERT_EQUAL(Index64(1), tree->activeVoxelCount());
+                        CPPUNIT_ASSERT(tree->isValueOn(ijk));
+                        //openvdb::tools::dilateVoxels(*tree, iter, openvdb::tools::NN_FACE_EDGE);
+                        openvdb::tools::dilateVoxels(*tree, iter, openvdb::tools::NN_FACE);
+                        //std::cerr << "Dilated to: " << tree->activeVoxelCount() << std::endl;
+                        //if (iter==1) {
+                        //    CPPUNIT_ASSERT_EQUAL(Index64(1 + 6 + 12), tree->activeVoxelCount());
+                        //}
+                        openvdb::tools::erodeVoxels( *tree, iter, openvdb::tools::NN_FACE_EDGE);
+                        CPPUNIT_ASSERT_EQUAL(Index64(1), tree->activeVoxelCount());
+                        CPPUNIT_ASSERT(tree->isValueOn(ijk));
+                    }
+                }
+            }
+        }
     }
-    
+#endif
+#if 0
     {//erode26
+        /// @todo Not implemented yet
         tree->clear();
         tree->setValue(openvdb::Coord(3,4,5), 1.0f);
-        openvdb::tools::dilateVoxels(*tree, 12, openvdb::tools::NN_FACE_EDGE_VERTEX);
-        openvdb::tools::erodeVoxels( *tree, 12, openvdb::tools::NN_FACE_EDGE_VERTEX);
+        openvdb::tools::dilateVoxels(*tree, 1, openvdb::tools::NN_FACE_EDGE_VERTEX);
+        CPPUNIT_ASSERT_EQUAL(Index64(1 + 6 + 12 + 8), tree->activeVoxelCount());
+        openvdb::tools::erodeVoxels( *tree, 1, openvdb::tools::NN_FACE_EDGE_VERTEX);
+        //openvdb::tools::dilateVoxels(*tree, 12, openvdb::tools::NN_FACE_EDGE);
+        //openvdb::tools::erodeVoxels( *tree, 12, openvdb::tools::NN_FACE_EDGE);
         CPPUNIT_ASSERT_EQUAL(1, int(tree->activeVoxelCount()));
         CPPUNIT_ASSERT(tree->isValueOn(openvdb::Coord(3,4,5)));
         }
-    */
+#endif
 }
 
 
@@ -860,15 +876,72 @@ void
 TestTools::testLevelSetAdvect()
 {
     // Uncomment sections below to run this (time-consuming) test
-    /*
-    const int dim = 64;//256
-    const openvdb::Vec3f center(0.35f, 0.35f, 0.35f);
+    using namespace openvdb;
+
+    const int dim = 128;
+    const Vec3f center(0.35f, 0.35f, 0.35f);
     const float radius = 0.15f, voxelSize = 1.0f/(dim-1);
+    const float halfWidth = 3.0f, gamma = halfWidth*voxelSize;
 
-    typedef openvdb::FloatGrid GridT;
-    typedef openvdb::Vec3fGrid VectT;
+    typedef FloatGrid GridT;
+    //typedef Vec3fGrid VectT;
 
-    */
+    {//test tracker::resize
+        GridT::Ptr grid = tools::createLevelSetSphere<GridT>(radius, center, voxelSize, halfWidth);
+        typedef tools::LevelSetTracker<GridT>  TrackerT;
+        TrackerT tracker(*grid);
+        tracker.setSpatialScheme(math::FIRST_BIAS);
+        tracker.setTemporalScheme(math::TVD_RK1);
+
+        ASSERT_DOUBLES_EXACTLY_EQUAL( gamma, grid->background());
+        ASSERT_DOUBLES_EXACTLY_EQUAL( halfWidth, tracker.getHalfWidth());
+
+        CPPUNIT_ASSERT(!tracker.resize());
+
+        {// check range of on values in a sphere w/o mask
+            tools::CheckRange<GridT, true, true, GridT::ValueOnCIter> c(-gamma, gamma);
+            tools::Diagnose<GridT> d(*grid);
+            std::string str = d.check(c);
+            //std::cerr << "Values out of range:\n" << str;
+            CPPUNIT_ASSERT(str.empty());
+            CPPUNIT_ASSERT_EQUAL(0, int(d.valueCount()));
+            CPPUNIT_ASSERT_EQUAL(0, int(d.failureCount()));
+        }
+        {// check norm of gradient of sphere w/o mask
+            tools::CheckNormGrad<GridT> c(*grid, 0.9f, 1.1f);
+            tools::Diagnose<GridT> d(*grid);
+            std::string str = d.check(c, false, true, false, false);
+            //std::cerr << "NormGrad:\n" << str;
+            CPPUNIT_ASSERT(str.empty());
+            CPPUNIT_ASSERT_EQUAL(0, int(d.valueCount()));
+            CPPUNIT_ASSERT_EQUAL(0, int(d.failureCount()));
+        }
+
+        CPPUNIT_ASSERT(tracker.resize(4));
+
+        ASSERT_DOUBLES_EXACTLY_EQUAL( 4*voxelSize, grid->background());
+        ASSERT_DOUBLES_EXACTLY_EQUAL( 4.0f, tracker.getHalfWidth());
+
+        {// check range of on values in a sphere w/o mask
+            const float g = gamma + voxelSize;
+            tools::CheckRange<GridT, true, true, GridT::ValueOnCIter> c(-g, g);
+            tools::Diagnose<GridT> d(*grid);
+            std::string str = d.check(c);
+            //std::cerr << "Values out of range:\n" << str;
+            CPPUNIT_ASSERT(str.empty());
+            CPPUNIT_ASSERT_EQUAL(0, int(d.valueCount()));
+            CPPUNIT_ASSERT_EQUAL(0, int(d.failureCount()));
+        }
+        {// check norm of gradient of sphere w/o mask
+            tools::CheckNormGrad<GridT> c(*grid, 0.4f, 1.1f);
+            tools::Diagnose<GridT> d(*grid);
+            std::string str = d.check(c, false, true, false, false);
+            //std::cerr << "NormGrad:\n" << str;
+            CPPUNIT_ASSERT(str.empty());
+            CPPUNIT_ASSERT_EQUAL(0, int(d.valueCount()));
+            CPPUNIT_ASSERT_EQUAL(0, int(d.failureCount()));
+        }
+    }
     /*
     {//test tracker
         GridT::Ptr grid = openvdb::tools::createLevelSetSphere<GridT>(radius, center, voxelSize);
@@ -885,8 +958,8 @@ TestTools::testLevelSetAdvect()
             tracker.track();
             fw("Tracker", 0, 0);
         }
-        }
     */
+
     /*
     {//test EnrightField
         GridT::Ptr grid = openvdb::tools::createLevelSetSphere<GridT>(radius, center, voxelSize);
@@ -1601,6 +1674,154 @@ TestTools::testPointScatter()
     }
 }
 
+////////////////////////////////////////
+
+void
+TestTools::testVolumeAdvect()
+{
+    using namespace openvdb;
+
+    Vec3fGrid velocity(Vec3f(1.0f, 0.0f, 0.0f));
+    typedef FloatGrid GridT;
+    typedef tools::VolumeAdvection<Vec3fGrid> AdvT;
+    typedef tools::Sampler<1> SamplerT;
+
+    {//test non-uniform grids (throws)
+        GridT::Ptr density0 = GridT::create(0.0f);
+        density0->transform().preScale(Vec3d(1.0, 2.0, 3.0));//i.e. non-uniform voxels
+        AdvT a(velocity);
+        CPPUNIT_ASSERT_THROW((a.advect<GridT, SamplerT>(*density0, 0.1f)), RuntimeError);
+    }
+
+    {// test spatialOrder and temporalOrder
+        AdvT a(velocity);
+
+        // Default should be SEMI
+        CPPUNIT_ASSERT_EQUAL(1, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(1, a.temporalOrder());
+        CPPUNIT_ASSERT(!a.isLimiterOn());
+
+        a.setIntegrator(tools::Scheme::SEMI);
+        CPPUNIT_ASSERT_EQUAL(1, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(1, a.temporalOrder());
+        CPPUNIT_ASSERT(!a.isLimiterOn());
+
+        a.setIntegrator(tools::Scheme::MID);
+        CPPUNIT_ASSERT_EQUAL(1, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(2, a.temporalOrder());
+        CPPUNIT_ASSERT(!a.isLimiterOn());
+
+        a.setIntegrator(tools::Scheme::RK3);
+        CPPUNIT_ASSERT_EQUAL(1, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(3, a.temporalOrder());
+        CPPUNIT_ASSERT(!a.isLimiterOn());
+            
+        a.setIntegrator(tools::Scheme::RK4);
+        CPPUNIT_ASSERT_EQUAL(1, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(4, a.temporalOrder());
+        CPPUNIT_ASSERT(!a.isLimiterOn());
+
+        a.setIntegrator(tools::Scheme::MAC);
+        CPPUNIT_ASSERT_EQUAL(2, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(2, a.temporalOrder());
+        CPPUNIT_ASSERT( a.isLimiterOn());
+
+        a.setIntegrator(tools::Scheme::BFECC);
+        CPPUNIT_ASSERT_EQUAL(2, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(2, a.temporalOrder());
+        CPPUNIT_ASSERT( a.isLimiterOn());
+        
+        a.setLimiter(tools::Scheme::NO_LIMITER);
+        CPPUNIT_ASSERT_EQUAL(2, a.spatialOrder());
+        CPPUNIT_ASSERT_EQUAL(2, a.temporalOrder());
+        CPPUNIT_ASSERT(!a.isLimiterOn());
+    }
+    
+    {//test RK4 advect without a mask
+        GridT::Ptr density0 = GridT::create(0.0f), density1;
+        density0->fill(CoordBBox(Coord(0),Coord(6)), 1.0f);
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord( 3,3,3)), 1.0f);
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord(24,3,3)), 0.0f);
+        CPPUNIT_ASSERT( density0->tree().isValueOn(Coord( 3,3,3)));
+        CPPUNIT_ASSERT(!density0->tree().isValueOn(Coord(24,3,3)));
+        
+        AdvT a(velocity);
+        a.setIntegrator(tools::Scheme::RK4);
+        for (int i=1; i<=240; ++i) {
+            density1 = a.advect<GridT, SamplerT>(*density0, 0.1f);
+            //std::ostringstream ostr;
+            //ostr << "densityAdvect" << "_" << i << ".vdb";
+            //std::cerr << "Writing " << ostr.str() << std::endl;
+            //openvdb::io::File file(ostr.str());
+            //openvdb::GridPtrVec grids;
+            //grids.push_back(density1);
+            //file.write(grids);
+            density0 = density1;
+        }
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord(3,3,3)), 0.0f); 
+        CPPUNIT_ASSERT(density0->tree().getValue(Coord(24,3,3)) > 0.0f);
+        CPPUNIT_ASSERT(!density0->tree().isValueOn(Coord( 3,3,3)));
+        CPPUNIT_ASSERT( density0->tree().isValueOn(Coord(24,3,3)));
+    }
+    {//test MAC advect without a mask
+        GridT::Ptr density0 = GridT::create(0.0f), density1;
+        density0->fill(CoordBBox(Coord(0),Coord(6)), 1.0f);
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord( 3,3,3)), 1.0f);
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord(24,3,3)), 0.0f);
+        CPPUNIT_ASSERT( density0->tree().isValueOn(Coord( 3,3,3)));
+        CPPUNIT_ASSERT(!density0->tree().isValueOn(Coord(24,3,3)));
+        
+        AdvT a(velocity);
+        a.setIntegrator(tools::Scheme::BFECC);
+        for (int i=1; i<=240; ++i) {
+            density1 = a.advect<GridT, SamplerT>(*density0, 0.1f);
+            //std::ostringstream ostr;
+            //ostr << "densityAdvect" << "_" << i << ".vdb";
+            //std::cerr << "Writing " << ostr.str() << std::endl;
+            //openvdb::io::File file(ostr.str());
+            //openvdb::GridPtrVec grids;
+            //grids.push_back(density1);
+            //file.write(grids);
+            density0 = density1;
+        }
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord(3,3,3)), 0.0f); 
+        CPPUNIT_ASSERT(density0->tree().getValue(Coord(24,3,3)) > 0.0f);
+        CPPUNIT_ASSERT(!density0->tree().isValueOn(Coord( 3,3,3)));
+        CPPUNIT_ASSERT( density0->tree().isValueOn(Coord(24,3,3)));
+    }
+    {//test advect with a mask
+        GridT::Ptr density0 = GridT::create(0.0f), density1;
+        density0->fill(CoordBBox(Coord(0),Coord(6)), 1.0f);
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord( 3,3,3)), 1.0f);
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord(24,3,3)), 0.0f);
+        CPPUNIT_ASSERT( density0->tree().isValueOn(Coord( 3,3,3)));
+        CPPUNIT_ASSERT(!density0->tree().isValueOn(Coord(24,3,3)));
+
+        BoolGrid::Ptr mask = BoolGrid::create(false);
+        mask->fill(CoordBBox(Coord(4,0,0),Coord(30,8,8)), true);
+        
+        AdvT a(velocity);
+        a.setGrainSize(0);
+        a.setIntegrator(tools::Scheme::MAC);
+        //a.setIntegrator(tools::Scheme::BFECC);
+        //a.setIntegrator(tools::Scheme::RK4);
+        for (int i=1; i<=240; ++i) {
+            density1 = a.advect<GridT, BoolGrid, SamplerT>(*density0, *mask, 0.1f);
+            //std::ostringstream ostr;
+            //ostr << "densityAdvectMask" << "_" << i << ".vdb";
+            //std::cerr << "Writing " << ostr.str() << std::endl;
+            //openvdb::io::File file(ostr.str());
+            //openvdb::GridPtrVec grids;
+            //grids.push_back(density1);
+            //file.write(grids);
+            density0 = density1;
+        }
+        CPPUNIT_ASSERT_EQUAL(density0->tree().getValue(Coord(3,3,3)), 1.0f);
+        CPPUNIT_ASSERT(density0->tree().getValue(Coord(24,3,3)) > 0.0f);
+        CPPUNIT_ASSERT(density0->tree().isValueOn(Coord( 3,3,3)));
+        CPPUNIT_ASSERT(density0->tree().isValueOn(Coord(24,3,3)));
+    }
+}//testVolumeAdvect
 
 ////////////////////////////////////////
 
@@ -2052,6 +2273,59 @@ TestTools::testClipping()
     }
 }
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+void
+TestTools::testPrune()
+{
+    /// @todo Add more unit-tests!
+    
+    using namespace openvdb;
+
+    const float value = 5.345f;
+    
+    FloatTree tree(value);
+    CPPUNIT_ASSERT_EQUAL(Index32(0), tree.leafCount());
+    CPPUNIT_ASSERT_EQUAL(Index32(1), tree.nonLeafCount()); // root node
+    CPPUNIT_ASSERT(tree.empty());
+    
+    tree.fill(CoordBBox(Coord(-10), Coord(10)), value, /*active=*/false);
+    CPPUNIT_ASSERT(!tree.empty());
+    
+    tools::prune(tree);
+    
+    CPPUNIT_ASSERT_EQUAL(Index32(0), tree.leafCount());
+    CPPUNIT_ASSERT_EQUAL(Index32(1), tree.nonLeafCount()); // root node
+    CPPUNIT_ASSERT(tree.empty());
+
+    /*
+    {// Bechmark serial prune
+        util::CpuTimer timer;
+        initialize();//required whenever I/O of OpenVDB files is performed!
+        io::File sourceFile("/usr/pic1/Data/OpenVDB/LevelSetModels/crawler.vdb");
+        sourceFile.open(false);//disable delayed loading
+        FloatGrid::Ptr grid = gridPtrCast<FloatGrid>(sourceFile.getGrids()->at(0));
+        const Index32 leafCount = grid->tree().leafCount();
+        
+        timer.start("\nSerial tolerance prune");
+        grid->tree().prune();
+        timer.stop();
+        CPPUNIT_ASSERT_EQUAL(leafCount, grid->tree().leafCount());
+    }
+    {// Bechmark parallel prune
+        util::CpuTimer timer;
+        initialize();//required whenever I/O of OpenVDB files is performed!
+        io::File sourceFile("/usr/pic1/Data/OpenVDB/LevelSetModels/crawler.vdb");
+        sourceFile.open(false);//disable delayed loading
+        FloatGrid::Ptr grid = gridPtrCast<FloatGrid>(sourceFile.getGrids()->at(0));
+        const Index32 leafCount = grid->tree().leafCount();
+
+        timer.start("\nParallel tolerance prune");
+        tools::prune(grid->tree());
+        timer.stop();
+        CPPUNIT_ASSERT_EQUAL(leafCount, grid->tree().leafCount());
+        }
+    */
+}
+
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
