@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -126,7 +126,6 @@ private:
     int mWheelPos;
     bool mShiftIsDown, mCtrlIsDown, mShowInfo;
     bool mInterrupt;
-    int mWinWidth, mWinHeight;
 #if GLFW_VERSION_MAJOR >= 3
     GLFWwindow* mWindow;
 #endif
@@ -813,7 +812,7 @@ ViewerImpl::render()
 
         int width, height;
 #if GLFW_VERSION_MAJOR >= 3
-        glfwGetWindowSize(mWindow, &width, &height);
+        glfwGetFramebufferSize(mWindow, &width, &height);
 #else
         glfwGetWindowSize(&width, &height);
 #endif
@@ -1087,13 +1086,13 @@ ViewerImpl::keyCallback(int key, int action)
 
     if (keyPress) {
         switch (key) {
-        case '1':
+        case '1': case GLFW_KEY_KP_1:
             toggleRenderModule(0);
             break;
-        case '2':
+        case '2': case GLFW_KEY_KP_2:
             toggleRenderModule(1);
             break;
-        case '3':
+        case '3': case GLFW_KEY_KP_3:
             toggleRenderModule(2);
             break;
         case 'c': case 'C':
@@ -1226,6 +1225,6 @@ ViewerImpl::toggleInfoText()
 
 } // namespace openvdb_viewer
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2015 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
